#!/usr/bin/env python3

"""Read and write applet xml configuration files.

Provide functionality allowing the dock applets configuration to be saved
and loaded.

Store the configuration in a specified XML file

The file will contain the following information:
    : a list of all pinned app's .desktop files
    : the indicator type (light or dark)
    : whether unpinned apps from all workspaces are to be displayed
    : whether an indicator for each open window is to be displayed


Also provide the ability to read an xml file which contains details
of apps which are difficult to match with their respective .desktop
files and which specifies the .desktop file to use
"""

# Copyright (C) 1997-2003 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author:
#     Robin Thompson

import xml.etree.ElementTree as ET
import sys
import os
import distro as distribution


def write_xml(filename, desktop_files, light_ind, show_all_apps, multi_ind,
              use_dock_win_list, win_from_cur_ws_only, win_switch_unity_style,
              panel_change_color, dock_panel_change_only, use_panel_act_list,
              active_icon_bg, fallback_bar_col, spacing, attention_type,
              popup_delay, saved_configs, pa_on_all_ws, dock_fixed_size,
              click_action, theme):
    """ Write the xml file using the specified information

    The xml file will be in e.g. the following format:

        <root>
            <pinned_apps>
                <desktop_file name="name1"/>
                <desktop_file name="name2"/>
                etc...
            </pinned_apps>
            <light_ind>int</>
            <show_all_apps>True or False</>
            <multi_ind>True or False</>
            <click_restore_last_active>True or False</>
            <win_from_cur_ws_only>True or False</>
            <panel_change_color>True or False</>
            <dock_panel_change_only>True or False</>
            ...
            ...

        </root>

    Args:
        filename : the filename to use. If the file already exists it will be
                   overwritten.
        desktop_files: a list containing the names of the applet's pinned app's
                       .desktop files e.g. ['pluma.desktop']
        light_ind : int - the indicator type e.g. light, dark, bar to be used
        show_all_apps : boolean - Whether or not unpinned apps from all
                        workspaces are to be shown
        multi_ind : Whether indicators for each of an app's open windows are to
                    be shown
        use_dock_win_list: : boolean : whether to use the applet's own window list
                                  for switching between an app's open windows,
                                  or to use Compiz thumbnail previews
        win_from_cur_ws_only: boolean - If True, indicators and window list
                              items will only be shown for windows on the
                              current workspace (new with V0.67)
        win_switch_unity_style : boolean - If True, first left-click on a dock-icon
                                 switches to the last focused window first, in
                                 case the app has multiple windows opened
        panel_change_color : boolean - Whether or not MATE panels are to change
                             color according to the desktop wallpaper (new
                             with V0.66)
        dock_panel_change_only : boolean - whether panel colour changes are to
                                 limited to only the panel containing the dock
                                 applet (new with V0.66)
        use_panel_act_list:  boolean - whether to display an app's actions in
                                 the panel right click menu, as opposed to
                                 using the popup action list
        active_icon_bg:      int - the type of active icon background e.g.
                                   gradient or fill
        fallback_bar_col:   a list of the rgb elements of a color to be used
                            for bar indicators when the theme highlight colour
                            cannot be read
        spacing : the amount of spacing (0-8) between apps in the dock
        attention_type : how a docked app notifies the user when an app requires
                         attention
        popup_delay : the delay before action/windows lists appear

        saved_configs : a list of tuples containing details of the pinned app
                        config. Each tuple contains the
                        following:
                            string - the config name
                            string - the workspace (if any) the config is to be
                                     automatically loaded for
                            string - a .desktop filename specifying a pinned app
                            string - another .desktop filename etc. etc.
        pa_on_all_ws : boolean - whether pinned apps appear on all workspaces or
                                 only the workspace where they were pinned

        dock_fixed_size : the fixed size (if any) that the dock is to maintain
        click_action : what the applet does when a running app's icon is clicked
        theme : int - the theme used by the dock

    Returns:
        Boolean - True if the file was written successfully, False otherwise

    """

    root = ET.Element("root")
    pa_el = ET.Element("pinned_apps")

    for df in desktop_files:
        df_el = ET.Element("desktop_file", name=df)
        pa_el.append(df_el)

    ind_el = ET.Element("ind_type", light="%d" % light_ind)
    sa_el = ET.Element("show_all", show_all="%s" % show_all_apps)
    mi_el = ET.Element("multi_ind", show_multi="%s" % multi_ind)
    uwl_el = ET.Element("use_win_list",
                        uwl="%s" % use_dock_win_list)
    wcw_el = ET.Element("win_from_cur_ws_only",
                        wcw="%s" % win_from_cur_ws_only)
    wsu_el = ET.Element("win_switch_unity_style",
                        wsu="%s" % win_switch_unity_style)
    pcc_el = ET.Element("panel_change_color", pcc="%s" % panel_change_color)
    dcc_el = ET.Element("dock_panel_change_only",
                        dcc="%s" % dock_panel_change_only)
    pal_el = ET.Element("panel_act_list",
                        pal="%s" % use_panel_act_list)
    abg_el = ET.Element("bg_type", type="%d" % active_icon_bg)

    fbc_el = ET.Element("fallback_bar_col")
    for col in fallback_bar_col:
        col_el = ET.Element("col", value="%s" % col)
        fbc_el.append(col_el)

    spc_el = ET.Element("spacing", spc="%d" % spacing)
    att_el = ET.Element("attention_type", type="%d" % attention_type)
    pdy_el = ET.Element("popup_delay", delay="%d" % popup_delay)

    sc_el = ET.Element("saved_configs")
    for config in saved_configs:
        cfg_el = ET.Element("config")
        cnm_el = ET.Element("config_name", name=config[0])
        cws_el = ET.Element("workspace_name", workspace=config[1])
        cpa_el = ET.Element("pinned_apps")
        for loop in range(2, len(config)):
            df = config[loop]
            df_el = ET.Element("desktop_file", name=df)
            cpa_el.append(df_el)

        cfg_el.append(cnm_el)
        cfg_el.append(cws_el)
        cfg_el.append(cpa_el)

        sc_el.append(cfg_el)

    paoaws_el = ET.Element("pinned_apps_all_workspaces", paoaws="%s" % pa_on_all_ws)

    dfs_el = ET.Element("dock_fixed_size", dfs="%d" % dock_fixed_size)

    ct_el = ET.Element("click_action", ca="%d" % click_action)

    theme_el = ET.Element("theme", id="%d" % theme)

    root.append(pa_el)
    root.append(ind_el)
    root.append(sa_el)
    root.append(mi_el)
    root.append(uwl_el)
    root.append(wcw_el)
    root.append(wsu_el)
    root.append(pcc_el)
    root.append(dcc_el)
    root.append(pal_el)
    root.append(abg_el)
    root.append(fbc_el)
    root.append(spc_el)
    root.append(att_el)
    root.append(pdy_el)
    root.append(sc_el)
    root.append(paoaws_el)
    root.append(dfs_el)
    root.append(ct_el)
    root.append(theme_el)

    try:
        ET.ElementTree(root).write(filename, xml_declaration=True)
    except FileNotFoundError:
        return False  # invalid file or path name

    return True


def read_xml(filename):
    """ Reads an xml file created using the write_xml method

    Args:
        filename - the filename to read.

    Returns:
        boolean : True if the file was read successfully, False otherwise

        A tuple containing the following:
            a list of the .desktop files in the file (i.e. the pinned apps)
            an integer - the indicator setting
            a boolean - the show_all_apps setting
            a boolean - the multiple indicators setting
            a boolean - the use window list setting
            a boolean - the show indicators and win list items from the current
                        workspace only setting
            a boolean - the change panel colour setting
            a boolean - the change dock panel color only setting
            a boolean - the use panel action list setting
            an integer - the active icon background type
            a list of the r,g and b values (as strings) of the fallback bar indicator colour
            an integer - the spacing between apps
            an integer - the delay before window and acion lists popup
            a list of the workspace indexes that pinned apps are pinned to
            a tuple containing the details of pinned app configs - format is the same used when
            saving...
            a boolean - the pinned apps on all workspaces setting
            an int    - the fixed size (if any) of the dock
            an int    - the type of action to perform when a running app's icon is clicked
            an int    - the theme being useed by the dock
    """

    try:
        root = ET.parse(filename)
    except FileNotFoundError:
        return [False]

    df_list = []
    pinned_apps = root.find("pinned_apps")
    if pinned_apps is not None:
        for df in pinned_apps:
            df_list.append(df.get("name"))

    # note - values may be missing from the config file e.g. if a new version
    # of the applet adds a new configuration settings. If this happens, we just
    # assume a default option rather than reporting an error

    ind_el = root.find("ind_type")
    if ind_el is not None:
        light_ind = int(ind_el.get("light"))
    else:
        light_ind = 0

    ufa_el = root.find("show_all")
    if ufa_el is not None:
        show_all = ufa_el.get("show_all") == "True"
    else:
        show_all = True

    mi_el = root.find("multi_ind")
    if mi_el is not None:
        multi_ind = mi_el.get("show_multi") == "True"
    else:
        multi_ind = False

    uwl_el = root.find("use_win_list")
    if uwl_el is not None:
        use_win_list = uwl_el.get("uwl") == "True"
    else:
        use_win_list = True

    crla_el = root.find("win_from_cur_ws_only")
    if crla_el is not None:
        win_from_cur_ws_only = crla_el.get("wcw") == "True"
    else:
        win_from_cur_ws_only = True

    wsu_el = root.find("win_switch_unity_style")
    if wsu_el is not None:
        win_switch_unity_style = wsu_el.get("wsu") == "True"
    else:
        win_switch_unity_style = False

    crla_el = root.find("panel_change_color")
    if crla_el is not None:
        panel_change_color = crla_el.get("pcc") == "True"
    else:
        panel_change_color = False

    crla_el = root.find("dock_panel_change_only")
    if crla_el is not None:
        dock_panel_change_only = crla_el.get("dcc") == "True"
    else:
        dock_panel_change_only = False

    pal_el = root.find("panel_act_list")
    if pal_el is not None:
        use_panel_act_list = pal_el.get("pal") == "True"
    else:
        use_panel_act_list = False

    abg_el = root.find("bg_type")
    if abg_el is not None:
        bg_type = int(abg_el.get("type"))
    else:
        bg_type = 0

    fallback_bar_col = []
    fbc_col = root.find("fallback_bar_col")
    if fbc_col is not None and (len(fbc_col) == 3):
        for col in fbc_col:
            fallback_bar_col.append(col.get("value"))
    else:
        fallback_bar_col = ["128", "128", "128"]

    spc_el = root.find("spacing")
    if spc_el is not None:
        spacing = int(spc_el.get("spc"))
    else:
        spacing = 0

    att_el = root.find("attention_type")
    if att_el is not None:
        attention_type = int(att_el.get("type"))
    else:
        attention_type = 0

    pdy_el = root.find("popup_delay")
    if pdy_el is not None:
        popup_delay = int(pdy_el.get("delay"))
    else:
        popup_delay = 1000

    saved_configs = []
    sc_el = root.find("saved_configs")
    if sc_el is not None:
        for config in sc_el:

            config_name = ""
            config_ws = ""
            cnm_el = config.find("config_name")
            if cnm_el is not None:
                config_name = cnm_el.get("name")
            cws_el = config.find("workspace_name")
            if cws_el is not None:
                config_ws = cws_el.get("workspace")

            conf = []
            conf.append(config_name)
            conf.append(config_ws)
            pinned_apps = config.find("pinned_apps")
            if pinned_apps is not None:
                for app in pinned_apps:
                    conf.append(app.get("name"))

            saved_configs.append(conf)

    paoaws_el = root.find("pinned_apps_all_workspaces")
    if paoaws_el is not None:
        pinned_apps_all_workspaces = paoaws_el.get("paoaws") == "True"
    else:
        pinned_apps_all_workspaces = True

    dfs_el = root.find("dock_fixed_size")
    if dfs_el is not None:
        dock_fixed_size = int(dfs_el.get("dfs"))
    else:
        dock_fixed_size = -1  # the dock has no fixed size

    ct_el = root.find("click_action")
    if ct_el is not None:
        click_action = int(ct_el.get("ca"))
    else:
        click_action = 0

    theme_el = root.find("theme")
    if theme_el is not None:
        theme = int(theme_el.get("id"))
    else:
        theme = 0

    return [True, df_list, light_ind, show_all, multi_ind,
            use_win_list, win_from_cur_ws_only, win_switch_unity_style,
            panel_change_color, dock_panel_change_only,
            use_panel_act_list, bg_type, fallback_bar_col,
            spacing, attention_type, popup_delay, saved_configs,
            pinned_apps_all_workspaces, dock_fixed_size, click_action,
            theme]


def read_app_xml(filename):
    """ Reads the xml file containing the list of hard to match apps

    Args:
        filename - the filename to read.

    Returns:
        boolean : True if the file was read successfully, False otherwise

        A list of tuples containing the following:
            a string - the app name (as identified by wnck)
            a string - the window class of the app (as identified by wnck)
            a string - the .desktop file to be used for this app

        Note: the list will only contain enries relating to the distro the app
        is running on
    """

    distro = distribution.name()
    release = distribution.version()
    did = distribution.codename()

    if (distro is None) or (distro == ""):
        return [False]

    try:
        tree = ET.parse(filename)
        root = tree.getroot()
    except FileNotFoundError:
        return [False]

    app_list = []
    for entry in root.findall("app"):
        add_it = False
        if (distro is not None) and (distro != ""):
            if distro == entry.find("distro").text:
                if (release is not None) and (release != ""):

                    target_rel = entry.find("release").text
                    if target_rel is None:
                        # if we specify a release but the distro doesn't have one, just assume a match
                        add_it = True
                    else:
                        add_it = release in target_rel

                else:
                    add_it = True

        if add_it:
            app_list.append([entry.find("name").text,
                            entry.find("class").text,
                            entry.find("desktop").text])
    return [True, app_list]


def main():
    """Main function.

    Debugging code can go here
    """

    print(os.path.dirname(sys.argv[0]))
    results = read_app_xml("src/app_match.xml")
    if results[0]:
        for app in results[1]:
            print("App name = %s" % app[0])
            print("App class = %s" % app[1])
            print("App desktop = %s" % app[2])
    else:
        print("could not read app_match.xml")

#    write_xml ("/home/robin/tmp/text.xml", ["thing.desktop","blah.desktop"], 99, False, True, False, False, True, False)
#    results = read_xml ("/home/robin/tmp/text.xml")
#    if results[0] == True:
#        for df in results[1]:
#            print ("Desktop file found: %s" %df)

#        print ("Use light ind = %d" %results[2])
#        print ("Show unpinned on all = %s" %results[3])
#        print ("Multi ind = %s" %results[4])
#        print ("Click restore all = %s" %results[5])
#        print ("pinned on all = %s" %results[6])
#        print ("panel change color = %s" %results[7])
#        print ("dock panel only = %s" %results[8])
#    else:
#        print ("Error reading file....")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import os.path
import time


def log_it(thing, newfile=False):

    """Provide a quick and dirty logging facility

    Args:
        thing   : the string to be written to the log file
        newfile :  boolean - if True the log file is created, if False it is appended to
    """

    filename = os.path.expanduser("~/tmp/log")
    if os.path.isdir(os.path.expanduser("~/tmp")):
        if newfile:
            thefile = open(filename, 'w')
        else:
            thefile = open(filename, 'a')

        thefile.write(time.strftime("%d %b %X: " + thing + "\n"))
        thefile.close()



#!/usr/bin/env python3

""" Helper classes for apps in the dock

    Provide a base class and descendants which will allow various types
    of indicators to be drawn onto a Cairo canvas provided by an app
    in the dock

    Provide a base class and descendants which will allow various types
    of backgrounds (e.g. gradient fill) to be drawn onto a Cairo canvas
    provided by an app in the dock

"""

# Copyright (C) 1997-2003 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author:
#     Robin Thompson

# do not change the value of this variable - it will be set during build
# according to the value of the --with-gtk3 option used with .configure
build_gtk2 = False

import gi

if build_gtk2:
    gi.require_version("Gtk", "2.0")
    gi.require_version("Wnck", "1.0")
else:
    gi.require_version("Gtk", "3.0")

gi.require_version("MatePanelApplet", "4.0")

from gi.repository import Gtk
from gi.repository import MatePanelApplet
from gi.repository import GdkPixbuf
from gi.repository import Gdk
import cairo
import math
import os


class IndicatorType:
    """Class to define the indicator types"""
    LIGHT = 0    # Light Circle
    DARK = 1     # Dark Circle
    NONE = 2
    TBAR = 3     # Theme Bar
    TCIRC = 4    # Circle drawn in theme colour
    TSQUARE = 5  # Square drawn in theme colour
    TTRI = 6     # Triangle drawn in theme colour
    TDIA = 7     # Diamond drawn in theme colour
    SUBWAY = 8   # Metro type


class IconBgType:
    """Class to define the icon background types"""
    GRADIENT = 0
    ALPHAFILL = 1
    UNITY = 2
    UNITY_FLAT = 3


# static list to hold the rgb colour elements to use when drawing
# and indicator and we can't get the theme highlight colour i.e. when
# using gtk2
fallback_ind_col = [0.9, 0.9, 0.9]


def get_theme_highlight_col(applet):
    """
        get the current theme's highlight colour (Gtk3) or the fallback colour
        (Gtk2)

    Args:
        applet : the dock applet
    :return:
        a tuple containing the r,g,b values (0-1.0) of the colors

    """

    if build_gtk2:
        return fallback_ind_col
    else:

        context = applet.get_style_context()

        sel_bg = context.lookup_color("theme_selected_bg_color")
        if sel_bg[0]:
            hcol = sel_bg[1]
            return [hcol.red, hcol.green, hcol.blue]
        else:
            # assume what is hopefully a decent looking highlight
            # colour - something a bit brighter (or maybe a lot darker)
            # than the background
            c_info = context.lookup_color("dark_bg_color")
            if c_info[0]:
                bgcol = c_info[1]
                return [(bgcol.red + 0.25) % 1,
                        (bgcol.green + 0.25) % 1,
                        (bgcol.blue + 0.25) % 1]
            else:
                # we don't even have a background colour, so....
                return fallback_ind_col


class IndicatorDrawer(object):
    """ Base class for drawing indicators

     Provide a base class which can be used for drawing various type of
     app indicators onto Cairo surfaces

     This class must not be instantiated and it will be an error to try
     and use this to draw indicators. Descendant classes will be implement
     their own drawing functions

     Attributes:
        _context : the cairo context to draw onto
        _size    : the size (width and height - assumes is square...) of the docked app
        _orient  : the orientation of the dock applet e.g. MatePanelApplet.AppletOrient.RIGHT
        _num_ind : the number of indicators to draw, may not be applicable to all indicators
        extra_s  : (static) amount of extra size (in pixels) the indicator requires of the docked_app

    Descendant classes can implement their own properties as necessary if they need more control
    over the drawing process (e.g. to set a specific indicator colour)

    """

    extra_s = 0  # default value for most indicators - those which require more must override this

    def __init__(self, context, size, orient, num_ind=0):
        """ Constructor

        Set attributes according to the constructor parameters

        """

        super().__init__()

        self._context = context
        self._size = size
        self._orient = orient
        self._num_ind = num_ind

    def draw(self):
        """
            Abstract method - descendants will implement this as required and return
                              _surface when drawing is completed
        """

        raise NotImplementedError("Must be implemented by Indicator subclasses")


class DefaultInd(IndicatorDrawer):
    """
        Base class for the two variants (light and dark) of the default indicator
    """

    def __init__(self, context, size, orient, num_ind):
        """
            Constructor - call the inherited constructor and do additional
            bits of setup
        """
        super().__init__(context, size, orient, num_ind)

        # set the cairo colour values of the inner and outer areas of the indicator
        self._col1 = [0.0, 0.0, 0.0]
        self._col2 = [0.0, 0.0, 0.0]

    def draw(self):
        """
                  Draw up to 4 indicators

              """

        if self._orient == MatePanelApplet.AppletOrient.RIGHT:
            ind_x = 2
            ind_y = (self._size - 4) / (self._num_ind + 1) + 2

        elif self._orient == MatePanelApplet.AppletOrient.LEFT:
            ind_x = self._size - 1
            ind_y = (self._size - 4) / (self._num_ind + 1) + 2

        elif self._orient == MatePanelApplet.AppletOrient.DOWN:
            ind_x = (self._size - 4) / (self._num_ind + 1) + 2
            ind_y = 2
        else:
            ind_x = (self._size - 4) / (self._num_ind + 1) + 2
            ind_y = self._size - 1

        this_ind = 1
        while this_ind <= self._num_ind:
            rad_patt = cairo.RadialGradient(ind_x, ind_y, 2,
                                            ind_x, ind_y, 4)

            rad_patt.add_color_stop_rgba(0, self._col1[0], self._col1[1],
                                         self._col1[2], 1)
            rad_patt.add_color_stop_rgba(1, self._col2[0], self._col2[1],
                                         self._col2[2], 0)

            self._context.set_source(rad_patt)
            self._context.arc(ind_x, ind_y, 6, 0, 2 * math.pi)

            if self._num_ind > 1:
                if (self._orient == MatePanelApplet.AppletOrient.RIGHT) or \
                        (self._orient == MatePanelApplet.AppletOrient.LEFT):
                    ind_y += (self._size - 6) / (self._num_ind + 1)
                else:
                    ind_x += (self._size - 6) / (self._num_ind + 1)

            this_ind += 1

            self._context.fill()


class DefaultLightInd(DefaultInd):
    """
        Class to draw the dock applet's default light indicator

    """

    def __init__(self, context, size, orient, num_ind):
        """
                 Constructor - call the inherited constructor and do additional
                 bits of setup
        """

        super().__init__(context, size, orient, num_ind)

        # cairo colour values of the inner and outer areas of the indicator
        self._col1 = [0.9, 0.9, 0.9]
        self._col2 = [0.0, 0.0, 0.0]


class DefaultDarkInd(DefaultInd):
    """
        Class to draw the dock applet's default dark indicator
    """

    def __init__(self, context, size, orient, num_ind):
        """
                 Constructor - call the inherited constructor and do additional
                 bits of setup
        """

        super().__init__(context, size, orient, num_ind)

        # cairo colour values of the inner and outer areas of the indicator
        self._col1 = [0.0, 0.0, 0.0]
        self._col2 = [0.9, 0.9, 0.9]


class BarInd(IndicatorDrawer):
    """
        Base class for the bar indicator, a filled rectangle

        Multiple indicators are not supported - the bar runs the full width
        (or height, depending on the applet orientation) of the context
        and merely indicates that an app is running, not how many windows
        it has open...

    """

    def __init__(self, context, size, orient):
        """

        """
        super().__init__(context, size, orient)

        # cairo colour values of the bar, will be overridden by descendant classes
        self._barcol = [0.0, 0.0, 0.0]

    def draw(self):
        """
            Draw the bar along the edge of the panel adjoining the screen

        """

        line_size = 1
        self._context.set_line_cap(cairo.LINE_CAP_SQUARE)
        self._context.set_line_width(line_size)

        self._context.set_source_rgb(self._barcol[0], self._barcol[1], self._barcol[2])
        if self._orient == MatePanelApplet.AppletOrient.RIGHT:
            rect = [[0.5, 0.5], [2.5, 0.5],
                    [2.5, self._size - 0.5], [0.5, self._size - 0.5]]

        elif self._orient == MatePanelApplet.AppletOrient.LEFT:
            rect = [[self._size - 2.5, 0.5], [self._size - 0.5, 0, 5],
                    [self._size - 0.5, self._size - 0.5], [self._size - 2.5, self._size - 0.5]]

        elif self._orient == MatePanelApplet.AppletOrient.DOWN:
            rect = [[0.5, 0.5], [self._size - 0.5, 0.5],
                    [self._size - 0.5, 2.5], [0.5, 2.5]]

        else:
            rect = [[0.5, self._size - 2.5], [self._size - 0.5, self._size - 2.5],
                    [self._size - 0.5, self._size - 0.5], [0.5, self._size - 0.5]]

        self._context.set_line_width(1)
        self._context.move_to(rect[0][0], rect[0][1])

        for point in rect:
            self._context.line_to(point[0], point[1])

        self._context.line_to(rect[0][0], rect[0][1])

        self._context.stroke_preserve()
        self._context.fill()


class ThemeBarInd(BarInd):
    """
        A bar indicator in which the bar is drawn in the current theme's
        highlight colour (Gtk3) or using a fallback colour (Gtk2)
    """

    def __init__(self, context, size, orient, applet):
        """
            Constructor - call the inherited constructor and do additional
            bits of setup

        Args (in addition to those specified in the base class)
            applet : the applet

        """
        super().__init__(context, size, orient)

        # set the bar color
        self._barcol = get_theme_highlight_col(applet)


class ThemeCircleInd(IndicatorDrawer):
    """
        Draws round indicators (up to 4) with the current theme's highlight colour
    """

    def __init__(self, context, size, orient, applet, num_ind=0):
        """
            Constructor - call the inherited constructor and do additional
            bits of setup

        Args (in addition to those specified in the base class)
            applet : the applet

        """
        super().__init__(context, size, orient, num_ind)

        # set the indicator color
        self._indcol = get_theme_highlight_col(applet)

    def draw(self):
        """
                  Draw up to 4 indicators

              """

        if self._orient == MatePanelApplet.AppletOrient.RIGHT:
            ind_x = 2
            ind_y = (self._size - 4) / (self._num_ind + 1) + 2

        elif self._orient == MatePanelApplet.AppletOrient.LEFT:
            ind_x = self._size - 2
            ind_y = (self._size - 4) / (self._num_ind + 1) + 2

        elif self._orient == MatePanelApplet.AppletOrient.DOWN:
            ind_x = (self._size - 4) / (self._num_ind + 1) + 2
            ind_y = 2
        else:
            ind_x = (self._size - 4) / (self._num_ind + 1) + 2
            ind_y = self._size - 2

        this_ind = 1
        while this_ind <= self._num_ind:

            self._context.set_source_rgb(self._indcol[0], self._indcol[1], self._indcol[2])
            self._context.set_line_width(1)
            self._context.arc(ind_x, ind_y, 2, 0, 2 * math.pi)
            self._context.close_path()
            self._context.fill()

            if self._num_ind > 1:
                if (self._orient == MatePanelApplet.AppletOrient.RIGHT) or \
                        (self._orient == MatePanelApplet.AppletOrient.LEFT):
                    ind_y += (self._size - 6) / (self._num_ind + 1)
                else:
                    ind_x += (self._size - 6) / (self._num_ind + 1)

            this_ind += 1


class ThemeSquareInd(IndicatorDrawer):
    """
        Draws square indicators (up to 4) with the current theme's highlight colour
    """

    def __init__(self, context, size, orient, applet, num_ind=0):
        """
            Constructor - call the inherited constructor and do additional
            bits of setup

        Args (in addition to those specified in the base class)
            applet : the applet

        """
        super().__init__(context, size, orient, num_ind)

        # set the indicator color
        self._indcol = get_theme_highlight_col(applet)

    def draw(self):
        """
                  Draw up to 4 indicators

              """

        line_size = 1
        ind_size = 3

        self._context.set_line_width(line_size)

        # work out the x&y coords of the top left of the first indicator
        if self._orient == MatePanelApplet.AppletOrient.RIGHT:
            ind_x = 1.5
            ind_y = (self._size - ind_size) / (self._num_ind + 1) + 0.5

        elif self._orient == MatePanelApplet.AppletOrient.LEFT:
            ind_x = self._size - ind_size - 0.5
            ind_y = (self._size - ind_size) / (self._num_ind + 1) + 0.5

        elif self._orient == MatePanelApplet.AppletOrient.DOWN:
            ind_x = (self._size - ind_size) / (self._num_ind + 1) + 0.5
            ind_y = 1.5
        else:
            ind_x = (self._size - ind_size) / (self._num_ind + 1) + 0.5
            ind_y = self._size - ind_size - 0.5

        self._context.set_line_width(line_size)

        this_ind = 1
        while this_ind <= self._num_ind:

            self._context.set_source_rgb(self._indcol[0], self._indcol[1], self._indcol[2])

            self._context.rectangle(ind_x, ind_y, ind_size, ind_size)
            self._context.stroke_preserve()
            self._context.fill()

            if self._num_ind > 1:
                if (self._orient == MatePanelApplet.AppletOrient.RIGHT) or \
                        (self._orient == MatePanelApplet.AppletOrient.LEFT):
                    ind_y += (self._size - 6) / (self._num_ind + 1)
                else:
                    ind_x += (self._size - 6) / (self._num_ind + 1)

            this_ind += 1


class ThemeDiaInd(IndicatorDrawer):
    """
        Draws diamond indicators (up to 4) with the current theme's highlight colour
    """

    def __init__(self, context, size, orient, applet, num_ind=0):
        """
            Constructor - call the inherited constructor and do additional
            bits of setup

        Args (in addition to those specified in the base class)
            applet : the applet

        """
        super().__init__(context, size, orient, num_ind)

        # set the indicator color
        self._indcol = get_theme_highlight_col(applet)

    def draw(self):
        """
                  Draw up to 4 indicators

              """

        line_size = 1
        ind_size = 4.0

        # self._context.set_line_cap(cairo.LINE_CAP_ROUND)
        self._context.set_line_width(line_size)

        # work out the x&y coords of the top centre of the first indicator
        if self._orient == MatePanelApplet.AppletOrient.RIGHT:
            ind_x = (ind_size / 2) + 0.5
            ind_y = (self._size - ind_size) / (self._num_ind + 1) + 0.5

        elif self._orient == MatePanelApplet.AppletOrient.LEFT:
            ind_x = self._size - (ind_size / 2) - 0.5
            ind_y = (self._size - ind_size) / (self._num_ind + 1)

        elif self._orient == MatePanelApplet.AppletOrient.DOWN:
            ind_x = (self._size - ind_size) / (self._num_ind + 1) + (ind_size / 2) + 0.5
            ind_y = 1.5
        else:
            ind_x = (self._size - ind_size) / (self._num_ind + 1) + (ind_size / 2) + 0.5
            ind_y = self._size - ind_size - 0.5

        this_ind = 1
        while this_ind <= self._num_ind:

            self._context.set_source_rgb(self._indcol[0], self._indcol[1], self._indcol[2])
            self._context.move_to(ind_x, ind_y)
            self._context.line_to(ind_x + (ind_size / 2), ind_y + (ind_size / 2))
            self._context.line_to(ind_x, ind_y + ind_size)
            self._context.line_to(ind_x - (ind_size / 2), ind_y + (ind_size / 2))
            self._context.line_to(ind_x, ind_y)
            self._context.stroke_preserve()
            self._context.fill()

            if self._num_ind > 1:
                if (self._orient == MatePanelApplet.AppletOrient.RIGHT) or \
                        (self._orient == MatePanelApplet.AppletOrient.LEFT):
                    ind_y += (self._size - 6) / (self._num_ind + 1)
                else:
                    ind_x += (self._size - 6) / (self._num_ind + 1)

            this_ind += 1


class ThemeTriInd(IndicatorDrawer):
    """
        Draws triangle indicators (up to 4) with the current theme's highlight colour
    """

    def __init__(self, context, size, orient, applet, num_ind=0):
        """
            Constructor - call the inherited constructor and do additional
            bits of setup

        Args (in addition to those specified in the base class)
            applet : the applet

        """
        super().__init__(context, size, orient, num_ind)

        # set the bar color
        self._indcol = get_theme_highlight_col(applet)

    def draw(self):
        """
                  Draw up to 4 indicators

              """

        line_size = 1
        ind_width = 4
        ind_height = 3

        self._context.set_line_width(line_size)
        self._context.set_source_rgb(self._indcol[0], self._indcol[1], self._indcol[2])

        # for each panel orientation we need to make sure the triangle is drawn with its
        # point facing the icon...
        if self._orient == MatePanelApplet.AppletOrient.RIGHT:
            ind_x = 0.5
            ind_y = (self._size - ind_width) / (self._num_ind + 1) + 0.5
            point1 = [ind_x, ind_y]
            point2 = [ind_x + ind_height, ind_y + ind_width / 2]
            point3 = [ind_x, ind_y + ind_width]

        elif self._orient == MatePanelApplet.AppletOrient.LEFT:
            ind_x = self._size - 0.5
            ind_y = (self._size - ind_width) / (self._num_ind + 1) + 0.5
            point1 = [ind_x, ind_y]
            point2 = [ind_x - ind_height, ind_y + ind_height / 2]
            point3 = [ind_x, ind_y + ind_width]

        elif self._orient == MatePanelApplet.AppletOrient.DOWN:
            ind_x = (self._size - ind_width) / (self._num_ind + 1) + 0.5
            ind_y = 0.5
            point1 = [ind_x, ind_y]
            point2 = [ind_x + ind_width / 2, ind_y + ind_height]
            point3 = [ind_x + ind_width, ind_y]
        else:

            ind_x = (self._size - ind_width) / (self._num_ind + 1) + 0.5
            ind_y = self._size - 0.5
            point1 = [ind_x, ind_y]
            point2 = [ind_x + ind_width / 2, ind_y - ind_height]
            point3 = [ind_x + ind_width, ind_y]

        this_ind = 1

        while this_ind <= self._num_ind:

            self._context.move_to(point1[0], point1[1])
            self._context.line_to(point2[0], point2[1])
            self._context.line_to(point3[0], point3[1])
            self._context.line_to(point1[0], point1[1])
            self._context.stroke_preserve()
            self._context.fill()

            if self._num_ind > 1:
                if (self._orient == MatePanelApplet.AppletOrient.RIGHT) or \
                        (self._orient == MatePanelApplet.AppletOrient.LEFT):
                    point1[1] += (self._size - 6) / (self._num_ind + 1)
                    point2[1] += (self._size - 6) / (self._num_ind + 1)
                    point3[1] += (self._size - 6) / (self._num_ind + 1)

                else:
                    point1[0] += (self._size - 6) / (self._num_ind + 1)
                    point2[0] += (self._size - 6) / (self._num_ind + 1)
                    point3[0] += (self._size - 6) / (self._num_ind + 1)

            this_ind += 1


class SubwayInd(IndicatorDrawer):
    """
        Indicator which mimics the Metro look

    """

    extra_s = 4  # this type of indicator requires extra space

    def __init__(self, context, size, orient, applet, num_ind, surface, active):
        """
        Args (additional):
            surface : the cairo surface the indicators are being drawn on
            active  : bool - whether or not the app is active

        """
        super().__init__(context, size, orient, num_ind)

        # set the color of the bar indicator for the first indicator
        self._barcol = get_theme_highlight_col(applet)
        self._surface = surface
        self._active = active

    def draw(self):
        """
        The first open window of an app is drawn as for ThemeBarInd.

        If more than one indicator is required, the way it is drawn depends
        on whether or not the app active. If so, the last three columns (or
        rows, depending on the applet orientation) of the cairo surface
        are copied and drawn to the right (or below) the app icon. If not
        active, the rightmost part of the bar is drawn in a darker colour

        """

        b_o = 0.5  # orgin of the bar in cairo units
        b_w = 2.0  # width of the bar in cairo units

        # draw the bar
        line_size = 1
        self._context.set_line_cap(cairo.LINE_CAP_SQUARE)
        self._context.set_line_width(line_size)

        self._context.set_source_rgb(self._barcol[0], self._barcol[1], self._barcol[2])

        # define the four points to the bar - the order of the points is
        # top left, top right, bottom right, bottom left
        if self._orient == MatePanelApplet.AppletOrient.RIGHT:
            rect = [[b_o, b_o], [b_w, b_o],
                    [b_w, self._size - b_o], [b_o, self._size - b_o]]

        elif self._orient == MatePanelApplet.AppletOrient.LEFT:
            rect = [[self._size - b_w, b_o], [self._size - b_o, b_o],
                    [self._size - b_o, self._size - b_o], [self._size - b_w, self._size - b_o]]

        elif self._orient == MatePanelApplet.AppletOrient.DOWN:
            rect = [[b_o, b_o], [self._size - b_o, b_o],
                    [self._size - b_o, b_w], [b_o, b_w]]

        else:
            rect = [[b_o, self._size - b_w], [self._size - b_o, self._size - b_w],
                    [self._size - b_o, self._size - b_o], [b_o, self._size - b_o]]

        self._context.move_to(rect[0][0], rect[0][1])
        for point in rect:
            self._context.line_to(point[0], point[1])

        self._context.line_to(rect[0][0], rect[0][1])

        self._context.stroke_preserve()
        self._context.fill()

        if self._num_ind > 1:

            if self._active:
                # copy the required area from the surface to other parts of the surface...
                if self._orient == MatePanelApplet.AppletOrient.DOWN or \
                   self._orient == MatePanelApplet.AppletOrient.UP:
                    sx = self._size - 4

                    dx = self._size + 1
                    dy = 0
                    dw = 3
                    dh = self._size + 1

                else:
                    sx = 0

                    dx = 0
                    dy = self._size + 1
                    dw = self._size + 1
                    dh = 3

                # copy the relevant part of the source surface to a new surface
                copy_surf = cairo.ImageSurface(cairo.FORMAT_ARGB32, dw, dh)
                copy_ctx = cairo.Context(copy_surf)
                copy_ctx.set_source_surface(self._surface, -sx, 0)
                copy_ctx.rectangle(0, 0, dw, dh)
                copy_ctx.fill()

                # now draw onto the main surface
                if self._orient == MatePanelApplet.AppletOrient.DOWN or \
                   self._orient == MatePanelApplet.AppletOrient.UP:
                    self._context.set_source_surface(copy_surf, dx, 0)
                else:
                    self._context.set_source_surface(copy_surf, 0, dy)

                self._context.rectangle(dx, dy, dw, dh)
                self._context.fill()
            else:
                # darken part of the bar and draw a seperator between both parts

                darken_size = 5
                self._context.set_operator(cairo.OPERATOR_CLEAR)
                self._context.set_source_rgba(0, 0, 0, 1)
                if self._orient == MatePanelApplet.AppletOrient.DOWN or \
                   self._orient == MatePanelApplet.AppletOrient.UP:
                    self._context.move_to(rect[1][0] - darken_size, rect[1][1])
                    self._context.line_to(rect[1][0] - darken_size, rect[2][1])
                else:
                    self._context.move_to(rect[2][0], rect[2][1] - darken_size)
                    self._context.line_to(rect[3][0], rect[2][1] - darken_size)

                self._context.stroke()
                self._context.set_operator(cairo.OPERATOR_OVER)

                # darken with semi transparent black rectangle
                self._context.set_source_rgba(0, 0, 0, 0.20)
                if self._orient == MatePanelApplet.AppletOrient.DOWN or \
                   self._orient == MatePanelApplet.AppletOrient.UP:
                    self._context.move_to(rect[1][0] - darken_size - 1, rect[0][1])
                    self._context.line_to(rect[1][0], rect[1][1])
                    self._context.line_to(rect[2][0], rect[2][1])
                    self._context.line_to(rect[1][0] - darken_size - 1, rect[3][1])
                    self._context.line_to(rect[1][0] - darken_size - 1, rect[0][1])
                else:
                    self._context.move_to(rect[2][0], rect[2][1] - darken_size - 1)
                    self._context.line_to(rect[2][0], rect[2][1])
                    self._context.line_to(rect[3][0], rect[2][1])
                    self._context.line_to(rect[3][0], rect[2][1] - darken_size - 1)
                    self._context.line_to(rect[2][0], rect[2][1] - darken_size - 1)

                self._context.stroke_preserve()
                self._context.fill()


def ind_extra_s(indtype):
    """ Convenience function for returning the extra space required by an indicator

    Args: IndicatorType : The type of indicator e.g. ThemeTriInd

    Returns : int

    """

    if indtype == IndicatorType.LIGHT:
        return DefaultLightInd.extra_s
    elif indtype == IndicatorType.DARK:
        return DefaultDarkInd.extra_s
    elif indtype == IndicatorType.TBAR:
        return ThemeBarInd.extra_s
    elif indtype == IndicatorType.TCIRC:
        return ThemeCircleInd.extra_s
    elif indtype == IndicatorType.TSQUARE:
        return ThemeSquareInd.extra_s
    elif indtype == IndicatorType.TTRI:
        return ThemeTriInd.extra_s
    elif indtype == IndicatorType.TDIA:
        return ThemeDiaInd.extra_s
    elif indtype == IndicatorType.SUBWAY:
        return SubwayInd.extra_s
    else:
        return 0


###########################################################################################


class BackgroundDrawer(object):
    """ Base class for drawing the background of dock apps

     Provide a base class which can be used for drawing various type of
     backgrounds onto Cairo surfaces

     This class must not be instantiated and it will be an error to try
     and use this to draw backgrounds. Descendant classes will implement
     their own drawing functions

     Attributes:
        _context : the cairo context to draw onto
        _size    : the size (width and height - assumes is square...) of the size
        _orient  : the orientation of the dock applet e.g. MatePanelApplet.AppletOrient.RIGHT


    Descendant classes can implement their own properties as necessary if they need more control
    over the drawing process (e.g. to set a specific indicator colour)

    """

    def __init__(self, context, size, orient):
        """ Constructor

        Set attributes according to the constructor parameters

        """

        super().__init__()

        self._context = context
        self._size = size
        self._orient = orient

    def draw(self):
        """
            Abstract method - descendants will implement this as required and return
                              _surface when drawing is completed
        """

        raise NotImplementedError("Must be implemented by ActiveBackgroundDrawer descendents")


class DefaultBackgroundDrawer(BackgroundDrawer):
    """
        Class to draw the default active background, a colour gradient based on the
        the average colour of the app's icon
    """
    def __init__(self, context, size, orient, r, g, b):
        """ Constructor ...

        Args (in addition to those of the base class):
            r : the red component of the average colour
            g : the green component of the average colour
            b : the blue component of the average colour

        """
        super().__init__(context, size, orient)

        self._red = r
        self._green = g
        self._blue = b

    def draw(self):
        """
            Do the actual drawing, based on the panel orientation
        """

        # draw a background gradient according to the applet orientation
        if self._orient == MatePanelApplet.AppletOrient.RIGHT:
            pattern = cairo.LinearGradient(0, 0, self._size, 0)
            pattern.add_color_stop_rgba(0.0, self._red, self._green, self._blue, 1)
            pattern.add_color_stop_rgba(1.0, self._red, self._green, self._blue, 0)
        elif self._orient == MatePanelApplet.AppletOrient.LEFT:
            pattern = cairo.LinearGradient(self._size, 0, 0, 0)
            pattern.add_color_stop_rgba(0.0, self._red, self._green, self._blue, 1)
            pattern.add_color_stop_rgba(1.0, self._red, self._green, self._blue, 0)
        elif self._orient == MatePanelApplet.AppletOrient.DOWN:
            pattern = cairo.LinearGradient(0, 0, 0, self._size)
            pattern.add_color_stop_rgba(0.0, self._red, self._green, self._blue, 1)
            pattern.add_color_stop_rgba(1.0, self._red, self._green, self._blue, 0)
        else:
            pattern = cairo.LinearGradient(0, self._size, 0, 0)
            pattern.add_color_stop_rgba(0.0, self._red, self._green, self._blue, 1)
            pattern.add_color_stop_rgba(1.0, self._red, self._green, self._blue, 0)

        self._context.rectangle(0, 0, self._size, self._size)
        self._context.set_source(pattern)
        self._context.fill()


class AlphaFillBackgroundDrawer(BackgroundDrawer):
    """
        Fills the background with a specified colour and opacity
    """

    def __init__(self, context, size, orient, r, g, b, a):
        """ Constructor ...

        Args (in addition to those of the base class):
            r : the red component of the bg colour
            g : the green component of the bg colour
            b : the blue component of the bg colour
            a : the alpha component

        """
        super().__init__(context, size, orient)

        self._red = r
        self._green = g
        self._blue = b
        self.alpha = a

    def draw(self):
        """
            Do the actual drawing
        """

        self._context.rectangle(0, 0, self._size, self._size)
        self._context.set_source_rgba(self._red, self._green, self._blue, self.alpha)
        self._context.fill()


class UnityFlatBackgroundDrawer(BackgroundDrawer):
    """
        A Unity type flat backgroud (i.e. without the 'button' look)

        Running apps have their background filled with a specified colour
        (which will be based on the icon)

        Contains an extra method to be called after the app's icon
        has been drawn which will add a highlight
    """

    bg_surf = None
    shine_surf = None
    surf_size = 0

    def __init__(self, context, size, orient, r, g, b, running, scale_factor):
        """ Constructor ...

        Args (in addition to those of the base class):
            r : the red component of the average colour
            g : the green component of the average colour
            b : the blue component of the average colour
            running : whether or not the app is running
            scale_factor : 1 = standard def, higher values means HiDpi
        """

        super().__init__(context, size, orient)

        self._red = r
        self._green = g
        self._blue = b

        self._running = running
        self._scale_factor = scale_factor

        # load the .svgs used to draw the background and the shine_surf if
        # they haven't already been loaded, or they need to be reloaded
        # at a different size
        if (UnityFlatBackgroundDrawer.bg_surf is None) or (UnityFlatBackgroundDrawer.surf_size != size):
            self.load_bg()
            self.load_shine()
            UnityFlatBackgroundDrawer.surf_size = size

    def load_shine(self):
        """
            Load the background shine and convert to a cairo surface
        """

        d, f = os.path.split(os.path.abspath(__file__))

        if self._scale_factor == 1:
            fn = "assets/unity_dock_shine_56.svg"
        else:
            fn = "assets/unity_dock_shine_168.svg"  # load hipi version
        pb = GdkPixbuf.Pixbuf.new_from_file_at_size("%s/%s" % (d, fn),
                                                    self._size * 0.875, self._size * 0.875)

        UnityFlatBackgroundDrawer.shine_surf = Gdk.cairo_surface_create_from_pixbuf(pb, 1, None)

    def load_bg(self):
        """
            Load the background and convert to a cairo surface
        """

        d, f = os.path.split(os.path.abspath(__file__))
        if self._scale_factor == 1:
            fn = "assets/unity_dock_bg_56.svg"
        else:
            fn = "assets/unity_dock_bg_168.svg"  # load hipi version
        pb = GdkPixbuf.Pixbuf.new_from_file_at_size("%s/%s" % (d, fn),
                                                    self._size * 0.875, self._size * 0.875)

        UnityFlatBackgroundDrawer.bg_surf = Gdk.cairo_surface_create_from_pixbuf(pb, 1, None)

    def draw(self):
        """
            Draw the background
        """

        offset = self._size / 16

        if self._running:
            self._context.set_source_rgb(self._red, self._green, self._blue)
            self._context.mask_surface(UnityFlatBackgroundDrawer.bg_surf, offset, offset)
            self._context.fill()

    def draw_shine(self):
        """
            Draw the shine
        """
        offset = self._size / 16
        self._context.set_source_surface(UnityFlatBackgroundDrawer.shine_surf, 4, 4)
        self._context.paint()


class UnityBackgroundDrawer(UnityFlatBackgroundDrawer):
    """
        A Unity type backgroud (i.e. Unity Flat bur with the 'button' look)

        Running apps have their background filled with a specified colour
        (which will be based on the icon)

    """

    edge_surf = None
    edge_size = 0

    def __init__(self, context, size, orient, r, g, b, running, scale_factor):
        """ Constructor ...

        Args (in addition to those of the base class):
            r : the red component of the average colour
            g : the green component of the average colour
            b : the blue component of the average colour
            running : whether or not the app is running
        """

        super().__init__(context, size, orient, r, g, b, running, scale_factor)

        if (UnityBackgroundDrawer.edge_surf is None) or (UnityBackgroundDrawer.edge_size != size):
            self.load_edge()
            UnityBackgroundDrawer.edge_size = size

    def load_edge(self):
        """
            Load the background shine and convert to a cairo surface
        """

        d, f = os.path.split(os.path.abspath(__file__))
        if self._scale_factor == 1:
            fn = "assets/unity_dock_edge_56.svg"
        else:
            fn = "assets/unity_dock_edge_168.svg"  # load hipi version

        pb = GdkPixbuf.Pixbuf.new_from_file_at_size("%s/%s" % (d, fn),
                                                    self._size * 0.875, self._size * 0.875)

        UnityBackgroundDrawer.edge_surf = Gdk.cairo_surface_create_from_pixbuf(pb, 1, None)

    def draw(self):
        """
            Draw the background
        """

        offset = self._size / 16

        if self._running:
            self._context.set_source_rgb(self._red, self._green, self._blue)
            self._context.mask_surface(UnityFlatBackgroundDrawer.bg_surf, offset, offset)
            self._context.fill()
        else:
            self._context.set_source_surface(UnityBackgroundDrawer.edge_surf, 4, 4)
            self._context.paint()



#!/usr/bin/env python3

"""Provide a dialog for the dock panel applet that allows
   a custom app launcher to be added to the dock.

   Note: This is primarily intended for apps that do not create
   a .desktop file, or get installed into non-standard locations

    Mimic the layout of the standard MATE custom launcher dialog,
    allowing the user to specify the following:
        App name
        The command used to launch the app
        The icon to be displayed in the dock
        A comment

    When the Ok button is clicked, a .desktop file will be created
    in the ~/.local/share/applications and its name will be in
    the format 'mda_<app_name>.desktop' to allow the applet
    to recognise and if necessary give priority to self
    created custom launchers
"""
# Copyright (C) 1997-2003 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author:
#     Robin Thompson

# do not change the value of this variable - it will be set during build
# according to the value of the --with-gtk3 option used with .configure
build_gtk2 = False

import gi

if build_gtk2:
    gi.require_version("Gtk", "2.0")
else:
    gi.require_version("Gtk", "3.0")

from gi.repository import Gtk, GdkPixbuf

import os


class DockCLWindow(Gtk.Window):
    """Class to provide the create custom launcher functionality

    Create and display the create custom launcher dialog

    Provide properties to get and set the custom launcher name,
    command, comment and icon
    """
    def __init__(self, ok_callback):
        """ Constructor for the custom launchr window

        Create the window and its contents and display them

        set the callback for the ok button press

        Args:
            ok_callback : the method to be called when the ok button is
                          is pressed

        """

        super().__init__(title=_("Create Launcher"))

        self.set_skip_taskbar_hint(True)
        self.__icon_filename = ""

        self.connect("delete-event", self.win_delete_event)

        # setup the window contents
        self.set_border_width(5)

        if build_gtk2:
            self.__hbox = Gtk.HBox()
            self.__vbox = Gtk.VBox()
        else:
            self.__hbox = Gtk.Box()
            self.__hbox.set_orientation(Gtk.Orientation.HORIZONTAL)
            self.__vbox = Gtk.Box()
            self.__vbox.set_orientation(Gtk.Orientation.VERTICAL)

        self.__hbox.set_spacing(2)
        self.__vbox.set_spacing(2)

        self.__btn_help = Gtk.Button(label=_("Help"), stock=Gtk.STOCK_HELP)
        self.__btn_help.connect("button-press-event",
                                self.help_btn_press)
        self.__btn_cancel = Gtk.Button(label=_("Cancel"), stock=Gtk.STOCK_CANCEL)
        self.__btn_cancel.connect("button-press-event",
                                  self.win_cancel_button_press)
        self.__btn_ok = Gtk.Button(label=_("Ok"), stock=Gtk.STOCK_OK)
        self.__btn_ok.connect("button-press-event", ok_callback)

        if build_gtk2:
            self.__hbox_btns = Gtk.HBox()
            self.__hbbx = Gtk.HButtonBox()
        else:
            self.__hbox_btns = Gtk.Box()
            self.__hbox_btns.set_orientation(Gtk.Orientation.HORIZONTAL)
            self.__hbbx = Gtk.ButtonBox()
            self.__hbbx.set_orientation(Gtk.Orientation.HORIZONTAL)

        self.__hbbx.set_spacing(4)
        self.__hbbx.set_layout(Gtk.ButtonBoxStyle.END)

        self.__hbbx.pack_start(self.__btn_cancel, False, False, 4)
        self.__hbbx.pack_end(self.__btn_ok, False, False, 4)

        if build_gtk2:
            self.__hbbx1 = Gtk.HButtonBox()
        else:
            self.__hbbx1 = Gtk.ButtonBox()
            self.__hbbx1.set_orientation(Gtk.Orientation.HORIZONTAL)

        self.__hbbx1.set_spacing(4)
        self.__hbbx1.set_layout(Gtk.ButtonBoxStyle.START)
        self.__hbbx1.pack_start(self.__btn_help, False, False, 4)

        self.__hbox_btns.pack_start(self.__hbbx1, False, False, 0)
        self.__hbox_btns.pack_end(self.__hbbx, False, False, 0)

        self.__btn_icon = Gtk.Button()
        self.__img_icon = Gtk.Image()
        self.__btn_icon.connect("button_press_event", self.img_button_press)
        self.__btn_icon.set_tooltip_text(_("Click to select an icon"))
        self.__btn_icon.add(self.__img_icon)

        if build_gtk2:
            self.__vbox1 = Gtk.VBox()
        else:
            self.__vbox1 = Gtk.Box()
            self.__vbox1.set_orientation(Gtk.Orientation.VERTICAL)

        self.__vbox1.set_spacing(4)
        self.__vbox1.pack_start(self.__btn_icon, False, False, 4)

        if build_gtk2:
            self.__table_layout = Gtk.Table(rows=4, columns=2,
                                            homogeneous=False)
        else:
            self.__table_layout = Gtk.Grid()
            self.__table_layout.set_column_spacing(2)
            self.__table_layout.set_row_spacing(2)

        self.__lbl_name = Gtk.Label()
        self.__lbl_name.set_use_markup(True)
        self.__lbl_name.set_label("<b>" + _("Name:") + "</b>")
        self.__lbl_name.set_alignment(1, 0.5)

        self.__entry_name = Gtk.Entry()

        if build_gtk2:
            self.__hbox_cmd = Gtk.HBox()
        else:
            self.__hbox_cmd = Gtk.Box()
            self.__hbox_cmd.set_orientation(Gtk.Orientation.HORIZONTAL)

        self.__hbox_cmd.set_spacing(2)

        self.__lbl_cmd = Gtk.Label()
        self.__lbl_cmd.set_use_markup(True)
        self.__lbl_cmd.set_label("<b>" + _("Command:") + "</b>")
        self.__lbl_cmd.set_alignment(1, 0.5)

        self.__entry_cmd = Gtk.Entry()
        self.__entry_cmd.set_width_chars(40)
        self.__btn_cmd = Gtk.Button(label=_("Browse..."))
        self.__btn_cmd.connect("button-press-event", self.cmd_button_press)
        self.__hbox_cmd.pack_start(self.__entry_cmd, True, True, 0)
        self.__hbox_cmd.pack_end(self.__btn_cmd, False, False, 0)

        self.__lbl_term = Gtk.Label()
        self.__lbl_term.set_use_markup(True)
        self.__lbl_term.set_label("<b>" + _("Run in terminal:") + "</b>")
        self.__lbl_term.set_alignment(1, 0.5)

        self.__cbtn_term = Gtk.CheckButton()
        self.__cbtn_term.set_alignment(1, 0.5)

        self.__lbl_comment = Gtk.Label()
        self.__lbl_comment.set_use_markup(True)
        self.__lbl_comment.set_label("<b>" + _("Comment:") + "</b>")

        self.__entry_comment = Gtk.Entry()

        self.__lbl_wm_class = Gtk.Label()
        self.__lbl_wm_class.set_use_markup(True)
        self.__lbl_wm_class.set_label("<b>" + _("Window Class") + "</b>")
        self.__entry_wm_class = Gtk.Entry()
        self.__entry_wm_class.set_sensitive(False)

        if build_gtk2:
            self.__table_layout.attach(self.__lbl_name, 0, 1, 0, 1,
                                       Gtk.AttachOptions.SHRINK,
                                       Gtk.AttachOptions.SHRINK,
                                       2, 2)
            self.__table_layout.attach(self.__entry_name, 1, 2, 0, 1,
                                       Gtk.AttachOptions.FILL |
                                       Gtk.AttachOptions.EXPAND,
                                       Gtk.AttachOptions.SHRINK,
                                       2, 2)

            self.__table_layout.attach(self.__lbl_cmd, 0, 1, 1, 2,
                                       Gtk.AttachOptions.SHRINK,
                                       Gtk.AttachOptions.SHRINK,
                                       2, 2)
            self.__table_layout.attach(self.__hbox_cmd, 1, 2, 1, 2,
                                       Gtk.AttachOptions.FILL |
                                       Gtk.AttachOptions.EXPAND,
                                       Gtk.AttachOptions.SHRINK,
                                       2, 2)

        # Code below can be uncommented if adding terminal apps to the dock
        # ever becomes a needed thing
        # self.__table_layout.attach(self.__lbl_term, 0, 1, 2, 3,
        #                           Gtk.AttachOptions.FILL | Gtk.AttachOptions.EXPAND, Gtk.AttachOptions.SHRINK,
        #                           2, 2)
        #
        # self.__table_layout.attach(self.__cbtn_term, 1, 2, 2, 3,
        #                           Gtk.AttachOptions.FILL | Gtk.AttachOptions.EXPAND, Gtk.AttachOptions.SHRINK,
        #                           2, 2)

            self.__table_layout.attach(self.__lbl_comment, 0, 1, 2, 3,
                                       Gtk.AttachOptions.SHRINK,
                                       Gtk.AttachOptions.SHRINK,
                                       2, 2)
            self.__table_layout.attach(self.__entry_comment, 1, 2, 2, 3,
                                       Gtk.AttachOptions.FILL |
                                       Gtk.AttachOptions.EXPAND,
                                       Gtk.AttachOptions.SHRINK,
                                       2, 2)

            self.__table_layout.attach(self.__lbl_wm_class, 0, 1, 3, 4,
                                       Gtk.AttachOptions.SHRINK,
                                       Gtk.AttachOptions.SHRINK,
                                       2, 2)
            self.__table_layout.attach(self.__entry_wm_class, 1, 2, 3, 4,
                                       Gtk.AttachOptions.FILL |
                                       Gtk.AttachOptions.EXPAND,
                                       Gtk.AttachOptions.SHRINK,
                                       2, 2)
        else:
            self.__table_layout.attach(self.__lbl_name, 0, 0, 1, 1)
            self.__table_layout.attach(self.__entry_name, 1, 0, 1, 1)
            self.__table_layout.attach(self.__lbl_cmd, 0, 1, 1, 1)
            self.__table_layout.attach(self.__hbox_cmd, 1, 1, 1, 1)
            self.__table_layout.attach(self.__lbl_comment, 0, 2, 1, 1)
            self.__table_layout.attach(self.__entry_comment, 1, 2, 1, 1)
            self.__table_layout.attach(self.__lbl_wm_class, 0, 3, 1, 1)
            self.__table_layout.attach(self.__entry_wm_class, 1, 3, 1, 1)

        self.__hbox.pack_start(self.__vbox1, False, False, 0)
        self.__hbox.pack_end(self.__table_layout, True, True, 0)

        self.__vbox.pack_start(self.__hbox, True, True, 0)
        self.__vbox.pack_start(Gtk.HSeparator(), False, False, 4)
        self.__vbox.pack_end(self.__hbox_btns, False, False, 0)

        self.add(self.__vbox)
        self.set_default_values()
        self.show_all()

    def set_default_values(self):
        """ Set the window to its default state

        Clear all text entry fields
        Set the icon to Gtk.STOCK_EXECUTE
        """

        self.__img_icon.set_from_stock(Gtk.STOCK_EXECUTE, Gtk.IconSize.DIALOG)
        self.__entry_comment.set_text("")
        self.__entry_cmd.set_text("")
        self.__entry_name.set_text("")
        self.__entry_wm_class.set_text("")

    def win_delete_event(self, widget, event, data=None):
        """Callback for the preferences window delete event

        Do not delete the window, hide it instead so that it can be shown again
        later if needed

        """

        self.hide()
        return True

    def win_cancel_button_press(self, widget, event):
        """Callback for the preferences window Cancel button press

        Hide the window
        """
        self.hide()

    def set_cmd(self, cmd):
        """ Set the command line used by the launcher

        Args:
            cmd : the command line to use
        """

        self.__entry_cmd.set_text(cmd)

    def get_cmd(self):
        """ Get the command line used by the launcher

        Returns:
            A string containing the command line
        """

        return self.__entry_cmd.get_text()

    command = property(get_cmd, set_cmd)

    def cmd_button_press(self, widget, event):
        """Callback for the browse commands button

        Show a FileChooserDialog to allow the user to select a command to
        be associated with the laucher
        """

        fdc_cmd = Gtk.FileChooserDialog(title=_("Choose an application..."),
                                        action=Gtk.FileChooserAction.OPEN)

        # set the working directory of the dialog
        cmd = self.get_cmd()
        if cmd is not None:
            path, filename = os.path.split(cmd)
            if path is not None:
                fdc_cmd.set_current_folder(path)

        btn_cancel = fdc_cmd.add_button(Gtk.STOCK_CANCEL,
                                        Gtk.ResponseType.CANCEL)
        fdc_cmd.add_button(Gtk.STOCK_OPEN, Gtk.ResponseType.OK)
        btn_cancel.grab_default()
        response = fdc_cmd.run()

        if response == Gtk.ResponseType.OK:
            self.set_cmd(fdc_cmd.get_filename())

        fdc_cmd.destroy()

    def set_icon_filename(self, filename):
        """ Set the filename of the icon to be used with the launcher
            and update the image on the icon selection button

        Args: filename  - the filename
        """

        pixbuf = GdkPixbuf.Pixbuf.new_from_file(filename)
        pixbuf = pixbuf.scale_simple(48, 48, GdkPixbuf.InterpType.BILINEAR)
        self.__img_icon.set_from_pixbuf(pixbuf)
        self.__icon_filename = filename

    def get_icon_filename(self):
        """ Get the filename of the icon to be used with the launcher

        Returns: A string containing the filename
        """

        return self.__icon_filename

    icon_filename = property(get_icon_filename, set_icon_filename)

    def img_button_press(self, widget, event):
        """ Callback for the icon button press

        Show a FileChooserDialog allowing the user to select an icon for the
        launcher
        """

        fdc_icon = Gtk.FileChooserDialog(title=_("Choose an application..."),
                                         action=Gtk.FileChooserAction.OPEN)

        ff_graphic = Gtk.FileFilter()
        ff_graphic.set_name(_("Image files"))
        ff_graphic.add_pattern("*.svg")
        ff_graphic.add_pattern("*.png")
        ff_graphic.add_pattern("*.SVG")
        ff_graphic.add_pattern("*.PNG")
        ff_graphic.add_pattern("*.xpm")
        ff_graphic.add_pattern("*.XPM")
        fdc_icon.add_filter(ff_graphic)

        # set the working directory of the dialog
        icon_fn = self.get_icon_filename()
        if icon_fn is not None:
            path, filename = os.path.split(icon_fn)
            if path is not None:
                fdc_icon.set_current_folder(path)

        btn_cancel = fdc_icon.add_button(Gtk.STOCK_CANCEL,
                                         Gtk.ResponseType.CANCEL)
        fdc_icon.add_button(Gtk.STOCK_OPEN, Gtk.ResponseType.OK)
        btn_cancel.grab_default()
        response = fdc_icon.run()

        if response == Gtk.ResponseType.OK:
            self.set_icon_filename(fdc_icon.get_filename())

        fdc_icon.destroy()

    def get_comment(self):
        """ Get the comment associated with the launcher

        Returns: string
        """

        return self.__entry_comment.get_text()

    def set_comment(self, comment):
        """ Set the comment associated with the launcher

        Args: comment  - a string containing the comment
        """

        self.__entry_comment.set_text(comment)

    comment = property(get_comment, set_comment)

    def get_name(self):
        """ Get the name associated with the launcher

        Returns: a string containing the name
        """

        return self.__entry_name.get_text()

    def set_name(self, name):
        """ Set the name associated with the launcher

        Args: name - a string
        """

        self.__entry_name.set_text(name)

    name = property(get_name, set_name)

    def get_wm_class(self):
        """ Get the window wm_class_name

        Returns: A string containing the wm_class_name
        """

        return self.__entry_wm_class.get_text()

    def set_wm_class(self, wm_class):
        """ Set the text of the launcher's wm_class entry widget

        Args:
            wm_class : The wm_class name
        """

        self.__entry_wm_class.set_text(wm_class)

    wm_class = property(get_wm_class, set_wm_class)

    def get_is_term(self):
        """ Gets whether or not the app is meant to be run in a terminal

        Returns: True if the app is to be run in a terminal, False otherwise
        """

        return self.__cbtn_term.get_active()

    def set_is_term(self, is_term):
        """ Sets whether or not the app is to be run in a terminal

        Args:
            is_term: boolean
        """

        self.__cbtn_term.set_active(is_term)

    is_terminal_app = property(get_is_term, set_is_term)

    def help_btn_press(self, widget, event):
        """ Event handler for the Help button press event

        Display an explanation of the usages of custom launchers in a dialog
        box
        """

        md = Gtk.MessageDialog(None, Gtk.DialogFlags.MODAL,
                               Gtk.MessageType.INFO, Gtk.ButtonsType.OK,
                               None)
        md.set_markup('<span size="x-large"><b>' + _("Custom Launchers") + '</b></span>')
        info_text = _("Custom launchers are an advanced feature meant to be used only with apps " +\
                    "that the dock does not recognise (i.e. they display the wrong name or icon). \n\n" + \
                    "Normally, this will only happen when the apps have been installed " + \
                    "to a non standard location within the file system, so for the vast majority of " + \
                    "apps this feature is not needed.\n\n" + \
                    "Note: if an app is running when a custom launcher is created for it, the app will " + \
                    "need to be closed and restarted for the dock to recognise it.")

        md.format_secondary_text(info_text)
        md.run()
        md.destroy()


def main():
    """main function - debug code can go here"""
    dclw = DockCLWindow(Gtk.main_quit)
    dclw.set_skip_taskbar_hint(False)
    dclw.name = "My test launcher"
    dclw.comment = "This is a comment"
    dclw.icon_filename = "/usr/share/pixmaps/ericWeb.png"
    dclw.command = "/usr/bin/gvim"
    dclw.wm_class = "Dock_custom_launcher.py"
    dclw.is_terminal_app = False
    Gtk.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""
    Provide notification when the desktop background changes to a new picture
    and, if necessary, change the color of the MATE panel(s) to the dominant
    color of the new image

"""
# Copyright (C) 1997-2003 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author:
#     Robin Thompson

# do not change the value of this variable - it will be set during build
# according to the value of the --with-gtk3 option used with .configure
build_gtk2 = False

import gi

if build_gtk2:
    gi.require_version("Gtk", "2.0")
else:
    gi.require_version("Gtk", "3.0")

from gi.repository import Gtk
from gi.repository import Gio

import cairo

import os
import threading

import dom_color
from collections import namedtuple
from time import sleep

from log_it import log_it as log_it

ChangeTup = namedtuple('ChangeTup', ['settings', 'ored', 'ogreen', 'oblue',
                                     'step_red', 'step_green', 'step_blue'])


class PanelColorChanger(object):
    """ Class to change the color of the MATE panel(s) to the dominant color of
        the wallpaper image

    Provide support for wallpaper images only - gradients, solid colours and
    slideshows will be ignored
    Change the panel color whenever the wallpaper image is changed
    Allow only a specific panel's color to be changed, rather than all panels

    """

    def __init__(self, update_callback=None):
        """ Init for the PanelColorChanger class

            Use Gio.Settings to monitor the MATE desktop wallpaper setting

            Args:
                updated_callback - a function to be called after the panel
                                   colors have been updated
        """

        super().__init__()

        self.update_cb = update_callback

        self.__red = self.__green = self.__blue = 0
        # will hold rgb of the dom color of the wallpaper

        self.__bg_settings = Gio.Settings.new("org.mate.background")

        self.__pf = self.__bg_settings.get_string("picture-filename")

        self.__panel_settings = Gio.Settings.new("org.mate.panel")

        self.__event_handler_id = 0

        self.__toplevel_id = ""

    def enable_color_change(self):
        """ Enable panel color changing

        Monitor the MATE wallpaper setting and create an event handler to be
        called whenever it changes """

        self.__event_handler_id = self.__bg_settings.connect("changed",
                                                             self.background_changed)

    def disable_color_change(self):
        """ Disable panel color changing

        Disconnect the event handler linked to wallpaper changes """

        self.__bg_settings.disconnect(self.__event_handler_id)

    def do_change_panel_color(self):
        """ Change the panel colour

        Call the event handler ourselves in order to change the panel colour
        """

        self.background_changed(None, "picture-filename")

    def set_single_panel(self, toplevel_id):
        """ Store the id of the panel which is the only one whose colour is
            to be changed

        Args:
            toplevel_id : the toplevel_id of the panel. If this is an empty
                          string all panels are to have their colour changed
        """

        self.__toplevel_id = toplevel_id

    def background_changed(self, settings, key):
        """ Callback for when the desktop wallpaper settings are changed
        """

        if key == "picture-filename":
            new_pf = self.__bg_settings.get_string("picture-filename")
            self.__pf = new_pf

            # we're only interested if the wallpaper is an image file
            if (new_pf is not None) and (new_pf != ""):
                pic_ext = os.path.splitext(new_pf)[1]
                if pic_ext.upper() != ".XML":

                    worker_thread = threading.Thread(target=self.change_panel_colors)
                    worker_thread.start()

    def get_dom_color(self):
        """ Get  the dominant color of the current desktop image
        """

        colstr = dom_color.get_dom_color(self.__pf)

        self.__red = int(colstr[0:2], 16)
        self.__green = int(colstr[2:4], 16)
        self.__blue = int(colstr[4:6], 16)

    def change_panel_colors(self):
        """ Change panel colors to the rgb of the current dominant color

            Change the colour smoothly over an interval of 0.5 seconds

        """

        self.get_dom_color()

        change_list = []  # initialise list of panels & settings we need to change

        # get the list of panels
        panel_list = self.__panel_settings.get_value("toplevel-id-list").unpack()
        for panel in panel_list:

            # do we want to change this panel?
            do_change = (self.__toplevel_id == "") or \
                        (self.__toplevel_id == panel)

            if do_change:
                # get the settings path for the current panel
                settings_path = "/org/mate/panel/toplevels/%s/background/" % panel

                # get this panel's settings
                psettings = Gio.Settings.new_with_path("org.mate.panel.toplevel.background",
                                                       settings_path)

                # get the panel's original colour rgb components
                colstr = psettings.get_string("color")

                # the color can be stored as either a set of rgba values or as
                # an rgb hex ...
                store_rgba = False
                store_rgb = False
                if colstr.startswith("rgba"):
                    store_rgba = True
                    colstrip = colstr[4:255]
                    colstrip = colstrip.strip("()")
                    cols = colstrip.split(",")
                    pr = int(cols[0])
                    pg = int(cols[1])
                    pb = int(cols[2])
                    po = float(cols[3])
                elif colstr.startswith("rgb"):
                    store_rgb = True
                    colstrip = colstr.strip("rgb()")
                    cols = colstrip.split(",")
                    pr = int(cols[0])
                    pg = int(cols[1])
                    pb = int(cols[2])
                else:
                    pr = int(colstr[1:3], 16)
                    pg = int(colstr[3:5], 16)
                    pb = int(colstr[5:7], 16)

                # we're going to change the panel's color in 25 discrete steps,
                # so get the difference we need to apply to each color
                # component each step
                if pr > self.__red:
                    rs = -(pr - self.__red)
                else:
                    rs = self.__red - pr

                if pg > self.__green:
                    gs = -(pg - self.__green)
                else:
                    gs = self.__green - pg

                if pb > self.__blue:
                    bs = -(pb - self.__blue)
                else:
                    bs = self.__blue - pb

                rs /= 25
                gs /= 25
                bs /= 25

                change_list.append(ChangeTup(settings=psettings, ored=pr,
                                             ogreen=pg, oblue=pb, step_red=rs,
                                             step_blue=bs, step_green=gs))

        # now do the colour change
        for loop in range(1, 25):
            for change_item in change_list:
                if loop == 1:
                    # make sure the panel in question is set to be a colour
                    change_item.settings.set_string("type", "color")

                # work out new rgb values for this interval
                new_red = int(change_item.ored +
                              (loop * change_item.step_red)) & 0xff
                new_blue = int(change_item.oblue +
                               (loop * change_item.step_blue)) & 0xff
                new_green = int(change_item.ogreen +
                                (loop * change_item.step_green)) & 0xff
                if store_rgba:
                    change_item.settings.set_string("color",
                                                    "rgba(%d,%d,%d,%0.6f)"
                                                    % (new_red, new_green,
                                                       new_blue, po))
                elif store_rgb:
                    change_item.settings.set_string("color",
                                                    "rgb(%d,%d,%d)"
                                                    % (new_red, new_green,
                                                       new_blue))
                else:
                    change_item.settings.set_string("color",
                                                    "#%.2x%.2x%.2x"
                                                    % (new_red, new_green,
                                                       new_blue))

            # all panels have been changed for this step, so pause for a bit
            sleep(0.02)

        # now that we've had a smooth transition, set panel colours to the
        # final value
        for change_item in change_list:
            if store_rgba:
                change_item.settings.set_string("color",
                                                "rgba(%d,%d,%d,%0.6f)"
                                                % (self.__red, self.__green,
                                                   self.__blue, po))
            else:
                change_item.settings.set_string("color",
                                                "#%.2x%.2x%.2x"
                                                % (self.__red, self.__green,
                                                   self.__blue))

        # finally, call the callback function
        if self.update_cb is not None:
            self.update_cb()

    def wallpaper_filename(self):
        """ Get the desktop wallpaper image filename
        """
        return self.__pf

    def panel_rgb(self):
        """ Get the rgb values of the colour we set the panel(s) to

            Returns:
                red, green, blue: integers

        """

        return self.__red, self.__green, self.__blue


class TestWindow(Gtk.Window):
    """Testing window for the color changer code"""

    def __init__(self):
        """Init for the Test window class

        Create the window and its contents
        """

        super().__init__(title=_("Dock color changer test"))

        self.__button = Gtk.Button(label=_("Close"), stock=Gtk.STOCK_CLOSE)
        self.__button.connect("button-press-event", self.win_button_press)
        self.connect("delete-event", self.win_delete_event)

        self.pcc = PanelColorChanger(self.refresh_ui)

        self.set_border_width(5)
        self.__vbox = Gtk.VBox()
        self.__vbox.set_spacing(2)
        self.__vbox1 = Gtk.VBox()
        self.__vbox1.set_spacing(2)
        self.__hbox = Gtk.HBox()
        self.__hbox.set_spacing(2)

        self.__hbx = Gtk.HButtonBox()
        self.__hbx.set_layout(Gtk.ButtonBoxStyle.END)
        self.__hbx.pack_start(self.__button, False, False, 4)

        self.__da = Gtk.DrawingArea()
        self.__da.set_size_request(128, 128)

        self.__da.connect("expose-event", self.da_expose_event)

        self.__lbl_desktop_img = Gtk.Label()
        self.__lbl_desktop_img.set_use_markup(True)

        self.__red = self.__green = self.__blue = 0
        self.__lbl_red = Gtk.Label(_("red:"))
        self.__lbl_green = Gtk.Label(_("green:"))
        self.__lbl_blue = Gtk.Label(_("blue:"))

        self.__vbox1.pack_start(self.__lbl_red, False, False, 2)
        self.__vbox1.pack_start(self.__lbl_green, False, False, 2)
        self.__vbox1.pack_start(self.__lbl_blue, False, False, 2)

        self.__hbox.pack_start(self.__da, False, False, 2)
        self.__hbox.pack_start(self.__vbox1, False, False, 2)

        self.__vbox.pack_start(self.__hbox, False, False, 2)
        self.__vbox.pack_start(self.__lbl_desktop_img, False, False, 0)
        self.__vbox.pack_end(self.__hbx, False, False, 5)
        self.add(self.__vbox)

    def da_expose_event(self, widget, event):
        """ Draw a rectangle filled withe dominant color of the image
        """

        # there are lots of drawing operations to be done, so do them to an
        # offscreen surface and when all is finished copy this to the window
        offscreen_surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 64, 64)
        ctx = cairo.Context(offscreen_surface)

        # convert the highlight values to their cairo equivalents
        self.__red, self.__green, self.__blue = self.pcc.panel_rgb()
        pfn = os.path.split(self.pcc.wallpaper_filename())[1]
        self.__lbl_desktop_img.set_markup("<b>%s</b>" % pfn)
        self.__lbl_red.set_text(_("red:") + " %s" % self.__red)
        self.__lbl_green.set_text(_("green:") + " %s" % self.__green)
        self.__lbl_blue.set_text(_("blue:") + " %s" % self.__blue)
        red = self.__red / 255
        green = self.__green / 255
        blue = self.__blue / 255

        ctx.rectangle(0, 0, 64, 64)
        ctx.set_source_rgb(red, green, blue)
        ctx.fill()

        screen_ctx = self.__da.window.cairo_create()
        screen_ctx.rectangle(event.area.x, event.area.y,
                             event.area.width, event.area.height)
        screen_ctx.clip()

        screen_ctx.set_source_surface(offscreen_surface, 0, 0)

        screen_ctx.paint()
        ctx = None
        screen_ctx = None

    def refresh_ui(self):
        self.__da.queue_draw()

    def win_delete_event(self, widget, event, data=None):
        """Callback for the about window delete event

        """

        Gtk.main_quit()
        return True

    def win_button_press(self, widget, event):
        """
        callback for the Ok button on the About dialog

        """

        Gtk.main_quit()


def main():
    """
    main function - debugging code goes here
    """

    test_win = TestWindow()
    test_win.show_all()
    Gtk.main()

    return


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
    Provide a window showing a list of an app's actions and allow the
    user to select one

    In addition to the actions defined in the app's .desktop file, a
    Pin/Unpin action will also be added as appropriate

    The window will function in a similar way to a tooltip i.e.
    it will appear when the mouse hovers over a dock icon and
    will disappear if the mouse moves away from the window or
    the dock applet.
"""

#
# Copyright (C) 1997-2003 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author:
#     Robin Thompson

# do not change the value of this variable - it will be set during build
# according to the value of the --with-gtk3 option used with .configure
build_gtk2 = False

import gi

if build_gtk2:
    gi.require_version("Gtk", "2.0")
    gi.require_version("Wnck", "1.0")
else:
    gi.require_version("Gtk", "3.0")
    gi.require_version("Wnck", "3.0")

gi.require_version("MatePanelApplet", "4.0")

from gi.repository import Gtk
from gi.repository import Wnck
from gi.repository import GdkPixbuf
from gi.repository import Gio
from gi.repository import Gdk
from gi.repository import GObject
from gi.repository import MatePanelApplet
from gi.repository import Pango

from dock_popup import DockPopup

from log_it import log_it as log_it

CONST_MAX_TITLE_WIDTH = 300         # Max width of the action text
CONST_SEP = "--------------------"  # text which denotes tree view item is a separator


class DockActionList(DockPopup):

    """ Descendent of Dockup to provide a list of a running app's
        open windows

    """

    def __init__(self, wnck_screen, panel_orient, scroll_adj):
        """
        create the window and its contents

        Args:
            wnck_screen: the wnck_screen of the applet
            panel_orient : the orientation of the panel
            scroll_adj   : an adjustment to be applied to the window position
                           because the dock has scrolling enabled
        """

        # call the base classes constructor
        DockPopup.__init__(self, wnck_screen, panel_orient, scroll_adj)

        # we use a treeview to list each action, so initialise it and its
        # liststore
        self.__tree_view = Gtk.TreeView()

        if not build_gtk2:
            self.__tree_view.set_valign(Gtk.Align.START)
            self.__tree_view.set_halign(Gtk.Align.START)
            self.__tree_view.hexpand = True
            self.__tree_view.vexpand = True

        self.__tree_view.set_headers_visible(False)

        # turn grid lines off, although they still seem to appear in some
        # themes e.g. Menta
        self.__tree_view.set_grid_lines(Gtk.TreeViewGridLines.NONE)

        self.__tree_view.set_hover_selection(True)

        # the liststore needs to contain an icon, the action text, and the
        # action itself
        self.__list_store = Gtk.ListStore(str, Gtk.Action, GdkPixbuf.Pixbuf)

        self.__icon_renderer = Gtk.CellRendererPixbuf()
        self.__title_renderer = Gtk.CellRendererText()

        # set default cell colours and padding
        self.__title_renderer.set_padding(2, 6)
        self.set_bg_col(32, 32, 32)

        # create columns for the treeview
        self.__col_icon = Gtk.TreeViewColumn("",
                                             self.__icon_renderer,
                                             pixbuf=2)

        self.__col_title = Gtk.TreeViewColumn("",
                                              self.__title_renderer,
                                              text=0)

        self.__col_title.set_sizing(Gtk.TreeViewColumnSizing.GROW_ONLY)
        self.__col_title.set_expand(True)

        self.__col_title.set_max_width(CONST_MAX_TITLE_WIDTH)
        self.__col_title.set_sizing(Gtk.TreeViewColumnSizing.GROW_ONLY)

        # add the columns
        self.__tree_view.set_model(self.__list_store)
        self.__tree_view.append_column(self.__col_icon)
        self.__tree_view.append_column(self.__col_title)
        self.__tree_view.set_row_separator_func(self.check_sep)

        # add the treeview to the window
        self.set_main_widget(self.__tree_view)

        self.__tree_view.connect("button-release-event", self.button_release)
        self.__tree_view.connect("size-allocate", self.treeview_allocate)

    def treeview_allocate(self, widget, allocation):
        """ Event handler for the tree view size-allocate event

        If the title column has expanded to its maximum width, ellipsize the
        title text...
        """

        if self.__col_title.get_width() == CONST_MAX_TITLE_WIDTH:
            self.__col_title.set_min_width(CONST_MAX_TITLE_WIDTH)
            self.__title_renderer.set_property("ellipsize",
                                               Pango.EllipsizeMode.END)

        self.__tree_view.get_selection().unselect_all()

    def set_colours(self, panel_colour):
        """ Sets the treeview colours (background, foreground and
            highlight) to match the window colours.

        Note : set_colours must have been called first so that
        the window colours are set correctly
        """

        DockPopup.set_colours(self, panel_colour)

        # set strings used to set widget colours - we can't change
        # the highlight colour, only foreground and background
        r, g, b = self.bg_col
        bg_str = "#%.2x%.2x%.2x" % (r, g, b)
        r, g, b = self.fg_col
        fg_str = "#%.2x%.2x%.2x" % (r, g, b)

        # now set the treeview colours
        self.__title_renderer.set_property("cell-background", bg_str)
        self.__title_renderer.set_property("foreground", fg_str)

        self.__icon_renderer.set_property("cell-background", bg_str)

    def get_num_rows(self):
        """ Returns the number of rows of data in the list store
        """

        return (self.__list_store.iter_n_children(None))

    def button_release(self, widget, event):
        """ Handler for the button release event

        If the middle or right mouse button was pressed, do nothing

        Otherwise, activate the selected item's action

        Hide the action list

        Args:

            widget : the widget the received the signal i.e. our treeview
            event  : the event parameters

        Returns:
            True: to stop any other handlers from being invoked
        """

        if event.button != 1:
            return False  # let other handlers run

        path, col, xrel, yrel = self.__tree_view.get_path_at_pos(event.x, event.y)
        sel_iter = self.__list_store.get_iter(path)

        action = self.__list_store.get_value(sel_iter, 1)
        title = self.__list_store.get_value(sel_iter, 0)
        if action is not None:
            self.hide()
            action.activate()

        return True

    def add_separator(self):
        """ Convenience method to add a separator to the list

        If there are no items currently in the list then the separator
        won't be added
        """

        if len(self.__list_store) > 0:
            self.add_to_list(CONST_SEP, None, False)

    def add_to_list(self, title, action, show_icon):
        """ Add an item to the action list

        Args:
            title - the title of the window or the action
            action - a GTK Action to be activated if the item is clicked
            show_icon - if True the app's icon will be shown alongside the
                        item in the list
        """

        if show_icon:
            app_icon = self.app_pb
        else:
            app_icon = None

        self.__list_store.append([title, action, app_icon])

    def clear_act_list(self):
        """ Clear the list of open windows """

        self.__list_store.clear()

    def win_button_press(self, widget, event):
        """ this is for debug puposes only"""
        Gtk.main_quit()

    def check_sep(self, model, iter, data=None):
        """ Check to see if the current row is to be displayed as a separator

            Args :
                model : the treeview model (will be self.__list_store)
                iter : the row in the model we're interested in
                data : user defined data

            Returns:
                Bool
        """

        title = model.get_value(iter, 0)
        return title == CONST_SEP


def main():
    """
    main function - debugging code goes here
    """

#    thewin = DockWinList()
#    thewin.set_app_name("Testing....")
#    thewin.add_to_list(None, False, "Win 1")
#    thewin.add_to_list(None, True, "Win 2 is active")
#    thewin.add_to_list(None, False, "Win 3")
#    thewin.show_all()

#    thewin.move(100, 110)
#    pos = thewin.get_position()
#    size = thewin.get_size()
#    print("pos %d %d" %(pos[0], pos[1]))
#    print("size %d %d" %(size[0], size[1]))
#    thewin.add_mouse_area(Gdk.Rectangle(pos[0]-15, pos[1]-15, size[0]+30, size[1]+30))
#    thewin.add_mouse_area(Gdk.Rectangle(0, 0, 48, 500))
#    thewin.add_mouse_area(Gdk.Rectangle(48, 110, 100, size[1]))
#    Gtk.main()
    return


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
""" Calculate the average color of an image

    Code adapted from from: https://github.com/ZeevG/python-dominant-image-colour
"""

import binascii
import struct

try:
    import Image
    import ImageDraw
except ImportError:
    from PIL import Image, ImageDraw


def get_dom_color(filename):
    image = Image.open(filename)
    image = image.resize((150, 150))      # optional, to reduce time

    colour_tuple = [None, None, None]
    for channel in range(3):
        # Get data for one channel at a time

        # in case of errors stop processing and return black as the
        # dominant colour
        try:
            pixels = image.getdata(band=channel)
        except ValueError:
            return "000000"

        values = []
        for pixel in pixels:
            values.append(pixel)

        colour_tuple[channel] = int(sum(values) / len(values))

    colour = binascii.hexlify(struct.pack('BBB', *colour_tuple)).decode('utf-8')
    return colour



#!/usr/bin/env python3

"""
    Window control library

    Provide function to minimise, restore, activate etc. Bamf.Windows

"""
# Copyright (C) 1997-2003 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author:
#     Robin Thompson

# do not change the value of this variable - it will be set during build
# according to the value of the --with-gtk3 option used with .configure
build_gtk2 = False

import gi

if build_gtk2:
    gi.require_version("Gtk", "2.0")
    gi.require_version("Wnck", "1.0")
else:
    gi.require_version("Gtk", "3.0")
    gi.require_version("Wnck", "3.0")

from gi.repository import Gtk, Wnck


def activate_win(win, event_time=None):
    """
    Activate the specified window

    Params:
        win : the Bamf.Window
        event_time : the time of the event which triggered this activation
    """
    if event_time is None:
        event_time = Gtk.get_current_event_time()

    wnck_win = Wnck.Window.get(win.get_xid())
    if wnck_win is not None:
        wnck_win.activate(event_time)


def minimise_win(win):
    """
        Minimise the specified window

        Params:
            win : the Bamf.Window
        """

    wnck_win = Wnck.Window.get(win.get_xid())
    if wnck_win is not None:
        wnck_win.minimize()


def close_win(win, event_time=0):
    """
    Close the specified window

    Params:
        win : the Bamf.Window
        event_time : the event time to passed to wnck_win.close
    """

    wnck_win = Wnck.Window.get(win.get_xid())
    if wnck_win is not None:
        wnck_win.close(event_time)


# we need to know what adjustments to apply when calculating minimize positions,
# when  the dock has scrolling enabled and the variable below defines
# a callback which the dock can set in order to provide this info
# the callback should return four integers, the x and y adjustments to
# be applied to the minimise position, and the max width and height of the
# scrollable area, and a string - the panel orient

adj_minimise_pos_cb = None


def set_minimise_target(win, x, y, width, height):
    """
        Set the on-screen rectangle that a specified Bamf.Window will visibly
        minimize to

    Params:
        win     : the Bamf.Window
        x       : the x coordinate of the top left corner
        y       : the y coordinate of the top left corner
        width   : the width of the rectangle
        height  : the height of the ractangle

    """

    wnck_win = Wnck.Window.get(win.get_xid())
    if wnck_win is None:
        return

    if adj_minimise_pos_cb is not None:
        final_x, final_y = adj_minimise_pos_cb(x, y)
    else:
        final_x = x
        final_y = y

    win_type = wnck_win.get_window_type()
    if ((win_type == Wnck.WindowType.NORMAL) or
        (win_type == Wnck.WindowType.DIALOG)) and \
            (wnck_win.is_skip_tasklist() is False):
        wnck_win.set_icon_geometry(final_x, final_y, width, height)


def get_wm_class_group_name(win):
    """
    Use wnck to get the wm_class name of a specified bamf.window

    Params:
        win     : the Bamf.Window

    returns:
        string : the wm_class_name, or None

    """
    wnck_win = Wnck.Window.get(win.get_xid())
    if wnck_win is None:
        return
    else:
        return wnck_win.get_class_group_name()


def get_icon_pb(win):
    """ Use Wnck to try and get a pixbuf of a specified window's icon

    Params: win : the Bamf.Window

    Returns:
        a Pixbuf of the app icon, or None if the window doesn't specify an icon

    """

    wnck_win = Wnck.Window.get(win.get_xid())
    if wnck_win is None:
        return None
    elif wnck_win.get_icon_is_fallback():
        return None

    return wnck_win.get_icon()



#!/usr/bin/env python3
"""
Provide an about dialog for the MATE dock applet applet

The dialog displays the following:
    applet name and version number
    licensing info (GPL3)
    hints and tips
    a close button
"""

# Copyright (C) 1997-2003 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author:
#     Robin Thompson
#

# do not change the value of this variable - it will be set during build
# according to the value of the --with-gtk3 option used with .configure
build_gtk2 = False

import gi

if build_gtk2:
    gi.require_version("Gtk", "2.0")
else:
    gi.require_version("Gtk", "3.0")

from gi.repository import Gtk, Pango


class AboutWindow(Gtk.Window):
    """Provides the About window

    """

    def __init__(self):
        """Init for the About window class

        Create the window and its contents
        """

        super().__init__(title=_("About Dock Applet"))

        self.set_position(Gtk.WindowPosition.CENTER_ALWAYS)
        self.set_skip_taskbar_hint(True)  # we don't want to be in the taskbar

        self.__btn_close = Gtk.Button(label=_("Close"), stock=Gtk.STOCK_CLOSE)
        self.__btn_close.connect("button-press-event", self.close_button_press)
        self.__btn_hints = Gtk.ToggleButton.new_with_label(_("Hints & Tips"))
        self.__btn_hints.connect("toggled", self.hints_button_toggled)
        self.__btn_license = Gtk.ToggleButton.new_with_label(_("License"))
        self.__btn_license.connect("toggled", self.license_button_toggled)

        self.connect("delete-event", self.win_delete_event)
        self.set_border_width(5)

        # use a notebook widget to display the various pages of info
        self.__nb = Gtk.Notebook()
        self.__nb.set_show_tabs(False)
        self.__nb.set_show_border(False)

        # create a container for the dialog and others for the pages of the notebook
        if build_gtk2:
            self.__vbox = Gtk.VBox()
            self.__vbox.set_spacing(2)
            self.__vbox_dflt = Gtk.VBox()
            self.__vbox_dflt.set_spacing(8)
            self.__vbox_license = Gtk.VBox()
            self.__vbox_license.set_spacing(8)
        else:
            self.__vbox = Gtk.Box()
            self.__vbox.set_orientation(Gtk.Orientation.VERTICAL)
            self.__vbox.set_spacing(4)

            self.__vbox_dflt = Gtk.Box()
            self.__vbox_dflt.set_orientation(Gtk.Orientation.VERTICAL)
            self.__vbox_dflt.set_spacing(8)

            self.__vbox_license = Gtk.Box()
            self.__vbox_license.set_orientation(Gtk.Orientation.VERTICAL)
            self.__vbox_license.set_spacing(8)

        if build_gtk2:
            self.__hbx = Gtk.HButtonBox()
        else:
            self.__hbx = Gtk.ButtonBox()
            self.__hbx.orientation = Gtk.Orientation.HORIZONTAL

        self.__hbx.set_layout(Gtk.ButtonBoxStyle.END)
        self.__hbx.set_spacing(4)
        self.__hbx.pack_start(self.__btn_hints, False, False, 4)
        self.__hbx.pack_start(self.__btn_license, False, False, 4)
        self.__hbx.pack_start(self.__btn_close, False, False, 4)

        self.__lbl_blank1 = Gtk.Label()
        self.__image = Gtk.Image()
        self.__image.set_from_stock(Gtk.STOCK_ABOUT, Gtk.IconSize.DIALOG)
        self.__lbl_title = Gtk.Label()
        self.__lbl_title.set_use_markup(True)
        if build_gtk2:
            gtk_ver = "GTK2"
        else:
            gtk_ver = "GTK3"

        self.__lbl_title.set_markup("<b>" + _("MATE Dock Applet") + "</b>")
        self.__lbl_ver = Gtk.Label("V" + "20.04.0 (" + gtk_ver + ")")
        self.__lbl_blank2 = Gtk.Label()

        self.__tb_gpl = Gtk.TextBuffer()
        self.__tag_size = self.__tb_gpl.create_tag("size")
        self.__tag_size.set_property("size-points", 9)
        iter_start = self.__tb_gpl.get_start_iter()

        self.__tb_gpl.insert_with_tags(iter_start,
                                       _("MATE Dock Applet is free software; you can redistribute it and/or modify it " +
                                       "under the terms of the GNU General Public Licence as published by the Free " +
                                       "Software Foundation; either version 3 of the Licence, or (at your option) " +
                                       "any later version. \n\nMATE Dock Applet is distributed in the hope that it " +
                                       "will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty " +
                                       "of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General " +
                                       "Public Licence for more details.\n\nYou should have received a copy of the " +
                                       "GNU General Public Licence along with the applet; if not, write to the Free " +
                                       "Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA " +
                                       "02110-1301 USA\n"),
                                       self.__tag_size)

        self.__tv_gpl = Gtk.TextView.new_with_buffer(self.__tb_gpl)
        self.__tv_gpl.set_wrap_mode(Gtk.WrapMode.WORD)
        self.__scrolled_win = Gtk.ScrolledWindow()

        self.__scrolled_win.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        self.__scrolled_win.add(self.__tv_gpl)

        self.__lbl_blurb1 = \
            Gtk.Label(_("A dock applet for the MATE desktop"))
        self.__lbl_blurb2 = \
            Gtk.Label(_("This program comes with ABSOLUTELY NO WARRANTY"))
        self.__lbl_blurb3 = Gtk.Label(_("and is distributed under the GNU General"))
        self.__lbl_blurb4 = Gtk.Label(_("Public License, version 3 or later"))

        if build_gtk2:
            self.__hbx_gpl = Gtk.HBox()
        else:
            self.__hbx_gpl = Gtk.Box()
            self.__hbx_gpl.set_orientation(Gtk.Orientation.HORIZONTAL)
            self.__hbx_gpl.set_spacing(0)

        self.__lbl_gpl1 = Gtk.Label(_("For details click"))
        self.__lb_gpl = \
            Gtk.LinkButton.new_with_label("http://www.gnu.org/licenses/gpl-3.0.html",
                                          _("here"))

        self.__hbx_gpl.pack_start(self.__lbl_gpl1, False, False, 0)
        self.__hbx_gpl.pack_start(self.__lb_gpl, False, False, 0)

        if build_gtk2:
            self.__vbox_hints = Gtk.VBox()
        else:
            self.__vbox_hints = Gtk.Box()
            self.__vbox_hints.set_orientation(Gtk.Orientation.VERTICAL)

        # create widgets where where the hints text can be displayed...
        self.__hints_scrolled_win = Gtk.ScrolledWindow()
        self.__hints_scrolled_win.set_policy(Gtk.PolicyType.NEVER,
                                             Gtk.PolicyType.AUTOMATIC)
        self.__tv_hints = Gtk.TextView()
        self.__tv_hints.set_wrap_mode(Gtk.WrapMode.WORD)
        self.__tv_hints.set_editable(False)
        self.__hints_text_buf = Gtk.TextBuffer()
        self.__tag_hint_bold = self.__hints_text_buf.create_tag("bold",
                                                                weight=Pango.Weight.BOLD,
                                                                scale=1.0,
                                                                justification=Gtk.Justification.CENTER)
        self.__tag_hint_normal = self.__hints_text_buf.create_tag("normal")
        self.__tag_hint_normal.set_property("size-points", 9)

        self.__tv_hints.set_buffer(self.__hints_text_buf)
        self.set_hints_text()
        self.__hints_scrolled_win.add(self.__tv_hints)

        self.__vbox_dflt.pack_start(self.__lbl_ver, False, False, 0)
        self.__vbox_dflt.pack_start(self.__lbl_blank2, False, False, 0)
        self.__vbox_license.pack_start(self.__scrolled_win, True, True, 2)

        self.__vbox_hints.pack_start(self.__hints_scrolled_win, True, True, 0)

        self.__pg_dflt = self.__nb.append_page(self.__vbox_dflt)
        self.__pg_license = self.__nb.append_page(self.__vbox_license)
        self.__pg_hints = self.__nb.append_page(self.__vbox_hints)

        self.__vbox.pack_start(self.__lbl_blank1, False, False, 2)
        self.__vbox.pack_start(self.__image, False, False, 2)
        self.__vbox.pack_start(self.__lbl_title, False, False, 2)
        self.__vbox.pack_start(self.__nb, True, True, 2)
        self.__vbox.pack_end(self.__hbx, False, False, 2)

        self.add(self.__vbox)

        self.set_size_request(-1, 300)

    def set_hints_text(self):
        """ Sets the text which is to be displayed
        """

        the_iter = self.__hints_text_buf.get_end_iter()

        if not build_gtk2:
            self.__hints_text_buf.insert_with_tags(the_iter,
                                                   _("Drag and drop data between applications") + "\n",
                                                   self.__tag_hint_bold)
            self.__hints_text_buf.insert_with_tags(the_iter, "\n" + _("To easily drag and drop data from one application " +
                                                   "to another, drag the data from the first application onto the dock " +
                                                   "icon of the second application. The dock will activate the second " +
                                                   "application's window, allowing the data to be dragged onto it. " +
                                                   "Note: the second application must already be running.") + "\n\n",
                                                   self.__tag_hint_normal)
            self.__hints_text_buf.insert_with_tags(the_iter,
                                                   _("Adding new applications to the dock") + "\n",
                                                   self.__tag_hint_bold)
            self.__hints_text_buf.insert_with_tags(the_iter, "\n" + _("Applications can be dragged and dropped from any menu " +
                                                   "applet (i.e. the Main Menu, Menu Bar, Advanced Menu, or Brisk Menu) " +
                                                   "directly onto the dock.") + "\n\n",
                                                   self.__tag_hint_normal)

        self.__hints_text_buf.insert_with_tags(the_iter,
                                               _("Activating apps with keyboard shortcuts") + "\n",
                                               self.__tag_hint_bold)
        self.__hints_text_buf.insert_with_tags(the_iter, "\n" + _("Holding down the <Super> (i.e. Windows) key and pressing " +
                                               "a number key will activate an app in the dock. For example, " +
                                               "pressing ""1"" will activate the first app, ""2"" the second etc. " +
                                               "Pressing ""0"" will activate the tenth app. To activate apps 11 to 20, " +
                                               "hold down the <Alt> key as well as <Super>.") + "\n\n",
                                               self.__tag_hint_normal)

        self.__hints_text_buf.insert_with_tags(the_iter,
                                               _("Opening a new instance of a running application") + "\n",
                                               self.__tag_hint_bold)
        self.__hints_text_buf.insert_with_tags(the_iter, "\n" + _("To quickly open a new instance of a running " +
                                               "application either hold down the <shift> key while clicking the " +
                                               "application's dock icon, or middle click on the icon." +
                                               "\n\nNote: this works for most, but not all, apps.") + "\n\n",
                                               self.__tag_hint_normal)

        self.__hints_text_buf.insert_with_tags(the_iter,
                                               "\n" + _("Window switching using the mouse wheel") + "\n",
                                               self.__tag_hint_bold)
        self.__hints_text_buf.insert_with_tags(the_iter,
                                               "\n" + _("To quickly switch between an application's open windows, move " +
                                               "the mouse cursor over the apps's dock icon and use the mouse " +
                                               "scroll wheel. This will activate and display each window in " +
                                               "turn, changing workspaces as necessary.") + "\n\n",
                                               self.__tag_hint_normal)

        self.__hints_text_buf.insert_with_tags(the_iter,
                                               "\n" + _("Panel colour changing") + "\n",
                                               self.__tag_hint_bold)
        self.__hints_text_buf.insert_with_tags(the_iter, "\n" + _("When the applet sets the panel colour for the " +
                                               "first time, the result may not look exactly as expected. This is " +
                                               "because the default opacity of custom coloured MATE panels is set " +
                                               "extremely low, so that the panel appears almost transparent.\n\nTo remedy " +
                                               "this, simply right click the panel, select Properties and adjust the " +
                                               "panel opacity as required."),
                                               self.__tag_hint_normal)

    def win_delete_event(self, widget, event, data=None):
        """Callback for the about window delete event

        Note: the window is not deleted, it is hidden instead so that it can
        be shown again if required later
        """

        self.hide()
        return True

    def close_button_press(self, widget, event):
        """
        callback for the Close button on the About dialog

        The window is hidden so that it can be shown again
        """

        self.hide()

    def license_button_toggled(self, widget):
        """
        callback for when the license button is toggled

        Show the license info if the license button is active, otherwise
        restore the main or credits info as appropriate

        Params:
            widget: the togglebuton

        """

        if self.__btn_license.get_active():

            self.__nb.set_current_page(self.__pg_license)
        else:
            if self.__btn_hints.get_active():
                self.__nb.set_current_page(self.__pg_hints)
            else:
                self.__nb.set_current_page(self.__pg_dflt)

    def hints_button_toggled(self, widget):
        """
        callback for when the credits button is toggled

        Show the credits info if the credits button is active, otherwise
        restore the main or license info as appropriate

        Params:
            widget: the togglebuton

        """

        if self.__btn_hints.get_active():
            self.__nb.set_current_page(self.__pg_hints)
        else:
            if self.__btn_license.get_active():
                self.__nb.set_current_page(self.__pg_license)
            else:
                self.__nb.set_current_page(self.__pg_dflt)


def main():
    """
    main function - debugging code goes here
    """

    about_dlg = AboutDialog()
    about_dlg.show_all()
    Gtk.main()

    return


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
    Provide a window showing a list of an app's open windows
    and also allow the user to select a window from the list and
    switch to it.

    Each item in the list will an indicator to show if the window
    is currently active, the window's title, and a close icon allowing
    the window to be closed.

    The window will function in a similar way to a tooltip i.e.
    it will appear when the mouse hovers over a dock icon and
    will disappear if the mouse moves away from the window or
    the dock applet.
"""

#
# Copyright (C) 1997-2003 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author:
#     Robin Thompson

# do not change the value of this variable - it will be set during build
# according to the value of the --with-gtk3 option used with .configure
build_gtk2 = False

import gi

if build_gtk2:
    gi.require_version("Gtk", "2.0")
    gi.require_version("Wnck", "1.0")
else:
    gi.require_version("Gtk", "3.0")
    gi.require_version("Wnck", "3.0")

gi.require_version("MatePanelApplet", "4.0")
gi.require_version("Bamf", "3")

from gi.repository import Gtk
from gi.repository import Wnck
from gi.repository import GdkPixbuf
from gi.repository import Gdk
from gi.repository import Pango
from gi.repository import Bamf

import os
import cairo
import tempfile
from time import sleep

from dock_popup import DockPopup
import window_control

from log_it import log_it as log_it

CONST_CLOSE_ICON_SIZE = 16
CONST_SEP = "--------------------"  # text which denotes tree view item is a separator
CONST_MAX_TITLE_WIDTH = 400         # max width of the title column in pixels
CONST__ACTIVE_TEXT = "•"


class DockWinList(DockPopup):

    """ Descendant of Dockup to provide a list of a running app's
        open windows

    """

    def __init__(self, wnck_screen, panel_orient, scroll_adj):
        """
        create the window and its contents

        Args:
            wnck_screen: the wnck_screen of the applet
            panel_orient : the orientation of the panel
            scroll_adj   : an adjustment to be applied to the window position
                           because the dock has scrolling enabled
        """

        # call the base classes constructor
        DockPopup.__init__(self, wnck_screen, panel_orient, scroll_adj)

        self.menu = None

        self.__bg_str = ""
        self.__fg_str = ""

        # we use a treeview to list each window, so initialise it and its
        # liststore
        self.__tree_view = Gtk.TreeView()

        if not build_gtk2:
            self.__tree_view.set_valign(Gtk.Align.START)
            self.__tree_view.set_halign(Gtk.Align.START)
            self.__tree_view.hexpand = True
            self.__tree_view.vexpand = True

        self.__tree_view.set_headers_visible(False)

        # turn grid lines off, although they still seem to appear in some
        # themes e.g. Menta
        self.__tree_view.set_grid_lines(Gtk.TreeViewGridLines.NONE)

        self.__tree_view.set_hover_selection(True)

        # the list consists of open windows (click to select the window)
        # the liststore therefore needs to contain an active indictor,
        # window title/item text, a Bamf window, a GdxPixbuf
        # (an icon for the user to click to close the window,

        self.__list_store = Gtk.ListStore(str, str, GdkPixbuf.Pixbuf,
                                          Bamf.Window,
                                          GdkPixbuf.Pixbuf)

        self.__active_renderer = Gtk.CellRendererText()
        self.__icon_renderer = Gtk.CellRendererPixbuf()
        self.__title_renderer = Gtk.CellRendererText()
        self.__close_renderer = Gtk.CellRendererPixbuf()
        self.__close_renderer.set_alignment(1, 0.0)   # align to to topright of the cell

        # set default cell colours and padding
        self.__title_renderer.set_padding(2, 6)
        self.set_bg_col(32, 32, 32)

        # create columns for the treeview
        self.__col_icon = Gtk.TreeViewColumn("",
                                             self.__icon_renderer,
                                             pixbuf=4)

        self.__col_active = Gtk.TreeViewColumn("",
                                               self.__active_renderer,
                                               text=0)

        self.__col_active.set_sizing(Gtk.TreeViewColumnSizing.GROW_ONLY)
        self.__col_active.set_min_width(0)

        self.__col_title = Gtk.TreeViewColumn("",
                                              self.__title_renderer,
                                              text=1)
        self.__col_title.set_sizing(Gtk.TreeViewColumnSizing.GROW_ONLY)
        self.__col_title.set_expand(True)

        self.__col_title.set_max_width(CONST_MAX_TITLE_WIDTH)
        # assign an event handler to the title column so that we can use it
        # to display the active window title in bold text
        self.__col_title.set_cell_data_func(self.__title_renderer,
                                            self.draw_title)

        self.__col_title.set_sizing(Gtk.TreeViewColumnSizing.GROW_ONLY)

        self.__col_close = Gtk.TreeViewColumn("",
                                              self.__close_renderer,
                                              pixbuf=2)
        self.__col_close.set_sizing(Gtk.TreeViewColumnSizing.GROW_ONLY)
        self.__col_close.set_max_width(32)

        # add the columns
        self.__tree_view.set_model(self.__list_store)
        self.__tree_view.append_column(self.__col_icon)
        self.__tree_view.append_column(self.__col_title)
        self.__tree_view.append_column(self.__col_close)

        self.__tree_view.set_row_separator_func(self.check_sep)

        # add the treeview to the window
        self.set_main_widget(self.__tree_view)

        self.__tree_view.set_has_tooltip(True)
        self.__tree_view.connect("query-tooltip", self.query_tooltip)
        self.__tree_view.connect("button-release-event", self.button_release)
        self.__tree_view.connect("size-allocate", self.treeview_allocate)

        self.__pb_close = None
        self.__pb_active = None

    def create_close_pixbuf(self):
        """ Create a 'close' icon (based on the stock close icon) for use
            in the treeview
        """

        # create a pixbuf for holding the 'close' icon
        pb_close = self.render_icon(Gtk.STOCK_CLOSE, Gtk.IconSize.MENU, None)

        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32,
                                     CONST_CLOSE_ICON_SIZE,
                                     CONST_CLOSE_ICON_SIZE)
        ctx = cairo.Context(surface)
        Gdk.cairo_set_source_pixbuf(ctx, pb_close, 0, 0)
        ctx.paint()

        # we now need to copy the cairo surface to a pixbuf. The best way to do
        # this would be by calling GdkPixbuf.Pixbuf.new_from_data as in these
        #                                                 64)
        # comments. Unfortunately this function does not seem to be
        # introspectable (Gtk2) or not implemented yet (Gtk3) and therefore
        # doesn't work.
        #
        # self.__pb_close = GdkPixbuf.Pixbuf.new_from_data(surface.get_data(),
        #                                                 GdkPixbuf.Colorspace.RGB,
        #                                                 True, 8, pb_close.get_width(),
        #                                                 pb_close.get_height(),

        # Therefore we have to resort to writing the surface to a temporary
        # .png file and then loading it into our pixbuf ...

        handle, tempfn = tempfile.mkstemp()
        surface.write_to_png(tempfn)
        self.__pb_close = GdkPixbuf.Pixbuf.new_from_file(tempfn)
        os.remove(tempfn)

    def create_active_pixbuf(self):
        """ Create an active window icon (based on the stock forward icon) for
            use in the treeview

            See the comments in create-close-pixbuf
        """

        # create a pixbuf for holding the icon
        pb_active = self.render_icon(Gtk.STOCK_GO_FORWARD, Gtk.IconSize.MENU, None)

        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32,
                                     CONST_CLOSE_ICON_SIZE,
                                     CONST_CLOSE_ICON_SIZE)
        ctx = cairo.Context(surface)
        Gdk.cairo_set_source_pixbuf(ctx, pb_active, 0, 0)
        ctx.paint()

        handle, tempfn = tempfile.mkstemp()
        surface.write_to_png(tempfn)
        self.__pb_active = GdkPixbuf.Pixbuf.new_from_file(tempfn)
        os.remove(tempfn)

    def treeview_allocate(self, widget, allocation):
        """ Event handler for the tree view size-allocate event

        If the title column has expanded to its maximum width, ellipsize the
        title text...
        """

        if self.__col_title.get_width() == CONST_MAX_TITLE_WIDTH:
            self.__col_title.set_min_width(CONST_MAX_TITLE_WIDTH)
            self.__title_renderer.set_property("ellipsize",
                                               Pango.EllipsizeMode.END)

        self.__tree_view.get_selection().unselect_all()

    def set_colours(self, panel_colour):
        """ Sets the treeview colours (background, foreground and
            highlight) to match the window colours.

        Note : set_colours must have been called first so that
        the window colours are set correctly
        """

        DockPopup.set_colours(self, panel_colour)

        # set strings used to set widget colours - we can't change
        # the highlight colour, only foreground and background
        r, g, b = self.bg_col
        self.__bg_str = "#%.2x%.2x%.2x" % (r, g, b)
        r, g, b = self.fg_col
        self.__fg_str = "#%.2x%.2x%.2x" % (r, g, b)

        # now set the treeview colours
        self.__title_renderer.set_property("cell-background", self.__bg_str)
        self.__title_renderer.set_property("foreground", self.__fg_str)

        self.__active_renderer.set_property("cell-background", self.__bg_str)
        self.__active_renderer.set_property("foreground", self.__fg_str)

        self.__close_renderer.set_property("cell-background", self.__bg_str)
        self.__icon_renderer.set_property("cell-background", self.__bg_str)

    def query_tooltip(self, widget, x, y, keyboard_mode, tooltip):
        """ Handler for the query-tooltip event to determine whether or not
            to show the 'Close window' tooltip

            If the tooltip was triggered by the keyboard, don't do anything

            Get the mouse coords, and if the mouse if located over the close
            icon show the tooltip

        Args:
            widget - the widget that received the event i.e. our treeview
            x - mouse x coord
            y - mouse y coord
            keyboard_mode - was the tooltip triggered by the keyboard?
            tooltip - the tooltip object that will be displayed

        Returns:
            True to display the tooltip, False otherwise
        """

        if keyboard_mode:
            return False

        return False
#        path, col, xrel, yrel = self.__tree_view.get_path_at_pos(x, y)

#        if col == self.__col_close:
#            cell_area = self.__tree_view.get_cell_area(path, col)
#            if (x >= cell_area.x + cell_area.width  - CONST_CLOSE_ICON_SIZE) and \
#               (y <= cell_area.y + CONST_CLOSE_ICON_SIZE):
#                tooltip.set_text(_("Close window"))
#                return True
#            else:
#                return False
#        else:
#            return False

    def button_release(self, widget, event):
        """ Handler for the button release event

        If the middle or right mouse button was pressed, do nothing

        If the mouse button was released over the close icon, close the
        associated window

        If the mouse button was released over any other part of the tree view
        item, activate the associated  window

        Finally, hide the window list

        Args:

            widget : the widget the received the signal i.e. our treeview
            event  : the event parameters

        Returns:
            True: to stop any other handlers from being invoked
        """

        if event.button != 1:
            return False  # let other handlers run

        path, col, xrel, yrel = self.__tree_view.get_path_at_pos(event.x, event.y)
        sel_iter = self.__list_store.get_iter(path)

        win = self.__list_store.get_value(sel_iter, 3)
        if (win is None) and (action is None):
            # this will allow e.g. an item to be added which just contains
            # the name of app, which can then be clicked to launch the app
            self.hide()
            self.the_app.start_app()
            return True

        if col == self.__col_close:
            cell_area = self.__tree_view.get_cell_area(path, col)
            if (event.x >= cell_area.x + cell_area.width - CONST_CLOSE_ICON_SIZE) and \
               (event.y <= cell_area.y + CONST_CLOSE_ICON_SIZE):
                window_control.close_win(win, event.time)
                self.hide()
                return True

        if win is not None:
            # if the window to be activated is not on the current workspace,
            # switchto that workspace
            wnck_win = Wnck.Window.get(win.get_xid())
            wnck_aws = self.wnck_screen.get_active_workspace()
            wnck_ws = wnck_win.get_workspace()

            # the windows's current workspace can be None if it is pinned to all
            # workspaces or it is not on any at all...
            # (fix for https://bugs.launchpad.net/ubuntu/+source/mate-dock-applet/+bug/1550392 and
            # https://bugs.launchpad.net/ubuntu-mate/+bug/1555336 (regarding software updater))
            if (wnck_aws is not None) and (wnck_ws is not None) and \
               (wnck_aws != wnck_ws):
                    wnck_ws.activate(0)
                    sleep(0.01)
            wnck_win.activate(0)

            # set the active indicator on the newly activated window
            # first, reset the current active indicator in the list store
            for list_item in self.__list_store:
                list_item[0] = ""
                list_item[4] = None

            # now set active indicator on the current item
            self.__list_store.set_value(sel_iter, 0, CONST__ACTIVE_TEXT)
            self.__list_store.set_value(sel_iter, 4, self.__pb_active)

            return True

        return True

    def add_separator(self):
        """ Convenience method to add a separator to the list

        If there are no items currently in the list then the separator
        won't be added
        """

        if len(self.__list_store) > 0:
            self.add_to_list(False, CONST_SEP, None)

    def add_to_list(self, is_active, title, window):
        """ Add an item to the window list

        Args:
            is_active - True if this is a window and it is active,
                         False otherwise
            title - the title of the window or the action
            window - the wnck window relating to the app (can be None)
        """

        # set the active indicator
        if is_active:
            active_text = CONST__ACTIVE_TEXT
        else:
            active_text = ""

        if active_text != "":
            app_icon = self.__pb_active
        else:
            app_icon = None

        if window is None:
            close_icon = None
        else:
            close_icon = self.__pb_close

        self.__list_store.append([active_text, title,
                                  close_icon, window,
                                  app_icon])

    def clear_win_list(self):
        """ Clear the list of open windows """

        self.__list_store.clear()

    def win_button_press(self, widget, event):
        """ this is for debug puposes only"""
        Gtk.main_quit()

    def setup_list(self, win_on_cur_ws_only):
        """ Setup the app list

        Set the app name

        Re-create the close icon in case the icon theme has changed

        For every window the app has open add an entry containing the app icon,
        window title, an indicator if the window is the active window, and a
        close icon

        Args:
            win_on_cur_ws_only : boolean - whether to show only windows which
                                 are on the current workspace, or show windows
                                 for all workspaces

        """

        self.create_close_pixbuf()
        self.create_active_pixbuf()

        # reduce the size of the window - it will autosize to fit the contents
        if build_gtk2:
            self.__col_title.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
            self.__col_title.set_fixed_width(150)
            self.resize(100, 10)
            self.__col_title.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)

        win_list = self.the_app.get_windows()

        wnck_aws = self.wnck_screen.get_active_workspace()
        add_to_list = False
        for win in win_list:
            win_type = win.get_window_type()

            if (win_type == Bamf.WindowType.NORMAL) or (win_type == Bamf.WindowType.DIALOG):
                if win_on_cur_ws_only:
                    # only add if the window is on the current workspace
                    wnck_win = Wnck.Window.get(win.get_xid())
                    add_to_list = wnck_win.is_on_workspace(wnck_aws)
                else:
                    add_to_list = True

                if add_to_list:
                    is_active = (len(win_list) == 1) or \
                                (win == self.the_app.last_active_win)

                    self.add_to_list(is_active, win.get_name(), win)

    def mbutton_press(self, widget, event):
        """ this is for debug purposes only and demonstrates that menu.popup does
            not work with Gtk 2
        """

        self.menu = Gtk.Menu()
        menu_item = Gtk.MenuItem("A menu item")
        self.menu.append(menu_item)
        menu_item.show()

        self.menu.popup(None, None, None, event.button, event.time)

    def draw_title(self, column, cell_renderer, tree_model, tree_iter, data):
        title = tree_model.get_value(tree_iter, 1)
        if tree_model.get_value(tree_iter, 0) == CONST__ACTIVE_TEXT:
            cell_renderer.set_property('markup', "<b><i>%s</i></b>" % title)
        else:
            # if there is no window of action associatied with this item
            # it must be the name of a non running app. Format the title
            # differently if so...
            if (tree_model.get_value(tree_iter, 3) is None) and \
               (tree_model.get_value(tree_iter, 4) is None):
                cell_renderer.set_property('markup', "<b>%s</b>" % title)
            else:
                cell_renderer.set_property('markup', title)

    def check_sep(self, model, iter, data=None):
        """ Check to see if the current row is to be displayed as a separator

            Args :
                model : the treeview model (will be self.__list_store)
                iter : the roow in the model we're interested in
                data : user defined data

            Returns:
                Bool
        """

        title = model.get_value(iter, 1)
        return title == CONST_SEP


def main():
    """
    main function - debugging code goes here
    """

#    thewin = DockWinList()
#    thewin.set_app_name("Testing....")
#    thewin.add_to_list(None, False, "Win 1")
#    thewin.add_to_list(None, True, "Win 2 is active")
#    thewin.add_to_list(None, False, "Win 3")
#    thewin.show_all()

#    thewin.move(100, 110)
#    pos = thewin.get_position()
#    size = thewin.get_size()
#    print("pos %d %d" %(pos[0], pos[1]))
#    print("size %d %d" %(size[0], size[1]))
#    thewin.add_mouse_area(Gdk.Rectangle(pos[0]-15, pos[1]-15, size[0]+30, size[1]+30))
#    thewin.add_mouse_area(Gdk.Rectangle(0, 0, 48, 500))
#    thewin.add_mouse_area(Gdk.Rectangle(48, 110, 100, size[1]))
#    Gtk.main()
    return


if __name__ == "__main__":
    main()
